precision highp float; 
      	
uniform sampler2D uTexture;

uniform sampler2D uShadowTexture;

uniform int uShadowEnabled;
      	
uniform float uShininess;        //shininess
uniform vec4 uLightPosition;   //light position

uniform vec4 uLightAmbient;      //light ambient property
uniform vec4 uLightDiffuse;      //light diffuse property
uniform vec4 uLightSpecular;     //light specular property

uniform vec4 uAmbient;  //object ambient property
uniform vec4 uDiffuse;   //object diffuse property
uniform vec4 uSpecular;  //object specular property 

uniform int uTextureAvailable;     	
      	 				
varying vec3 vNormal;
varying vec3 vEyeVec;
varying vec4 vPositionFromLight; // shadow texture coord 

varying vec2 vTexCoord; 

// Recalculate the z value from the rgba
float unpackDepth(const in vec4 rgbaDepth) {
	const vec4 bitShift = vec4(1.0, 1.0/256.0, 1.0/(256.0*256.0), 1.0/(256.0*256.0*256.0));
	float depth = dot(rgbaDepth, bitShift); // Use dot() since the calculations is same
	return depth;
}   							
  
void main()                    		
{       
	//light direction: pointing at the origin
    vec3 L = normalize(uLightPosition.xyz);

    //vec3 L = normalize(uLightDirection);
    vec3 N = normalize(vNormal);

    //Lambert's cosine law
    float lambertTerm = dot(N,-L);

    //Ambient Term
    vec4 Ia = uLightAmbient * uAmbient;

    //Diffuse Term
    vec4 Id = vec4(0.0,0.0,0.0,1.0);

    //Specular Term
    vec4 Is = vec4(0.0,0.0,0.0,1.0);    

    if(lambertTerm > 0.0) //only if lambertTerm is positive
    {
		Id = uLightDiffuse * uDiffuse * lambertTerm; //add diffuse term

        vec3 E = normalize(vEyeVec);
        vec3 R = reflect(L, N);
        float specular = pow( max(dot(R, E), 0.0), uShininess);

        Is = (uLightSpecular * uSpecular) * specular; //add specular term
     }
     
    vec4 finalColor = (Ia + (Id + Is));
    finalColor.a = 1.0;
     
	if( uTextureAvailable == 1 ){
    	finalColor *= texture2D(uTexture, vTexCoord);
    	
    	vec3 shadowCoord = (vPositionFromLight.xyz/vPositionFromLight.w)/2.0 + 0.5;
  		vec4 rgbaDepth = texture2D(uShadowTexture, shadowCoord.xy);
  		
  		float depth = unpackDepth(rgbaDepth); // Recalculate the z value from the rgba
  		float visibility = (shadowCoord.z > depth + 0.0015) ? 0.7 : 1.0;
  		
  		gl_FragColor = vec4(finalColor.rgb * visibility, 1.0);
    }    	   	
    else if( uTextureAvailable == 1 ){
    	finalColor *= texture2D(uTexture, vTexCoord);
    } else{
    	gl_FragColor = finalColor;
    } 
}