const int BONE_INFLUENCE_COUNT = 4;
const int MAX_BONE_COUNT = 30; 

precision highp float;
precision highp int; 

uniform mat4 uMVP;
uniform mat4 uNormalMatrix;
uniform highp mat4 uBoneTransforms[MAX_BONE_COUNT];
uniform int uBoneCount; 

attribute highp vec4 aPosition;
attribute highp vec3 aNormal;  
attribute highp vec2 aTexCoord;
attribute highp vec4 aBoneIndex;
attribute highp vec4 aBoneWeight;

varying vec2 vTexCoord;
varying vec4 vNormal;
varying vec3 eyespaceNormal;

void main()                    
{                            
    vTexCoord = aTexCoord;
    
    mediump ivec4 indexOfBone = ivec4(aBoneIndex);
    highp vec4 weightOfBone = aBoneWeight; 
    
    highp vec4 vertex;
    mediump vec4 normal;
    
    if (uBoneCount > 0) {
        highp mat4 mat = uBoneTransforms[indexOfBone.x] * weightOfBone.x 
        	+ uBoneTransforms[indexOfBone.y] * weightOfBone.y; 
        		+ uBoneTransforms[indexOfBone.z] * weightOfBone.z
        			+ uBoneTransforms[indexOfBone.w] * weightOfBone.w;
        
        //for (int i = 1; i < uBoneCount; i++) {
        //    // rotate to use indexOfBone.x in the loop
        //    indexOfBone = indexOfBone.yzwx;
        //    weightOfBone = weightOfBone.yzwx;
        //    if (weightOfBone.x > 0.0) {
        //        mat = mat + (uBoneTransforms[indexOfBone.x] * weightOfBone.x);
        //    }
        //     
        //}
        
        // resulting position after applying skinning
        vertex = aPosition * mat;
        normal = mat * vec4(aNormal, 1.0);
    } else {
        vertex = aPosition;
        normal = vec4(aNormal,1.0);        
    }   
    
    //normal = uNormalMatrix * vec4(normal,1.0);
    vNormal = normal; 
    
    
    // normal
    eyespaceNormal = vec3(uNormalMatrix * normal);
              
    gl_Position = uMVP * vertex;    
}  