precision mediump float; 
      	 				
uniform sampler2D uTexture;  

uniform mediump vec4 uDiffuse;
uniform mediump vec4 uAmbient;
uniform mediump vec4 uSpecular; 
uniform mediump float uShinness;   
uniform int uLightOn;   	 							

varying vec2 vTexCoord;    
varying vec4 vNormal;
varying vec3 vEyespaceNormal;  
varying vec3 vLightDir;	 
varying vec3 vEyeVec;  							
  
void main()                    		
{        
	// base colour                      	
    vec4 texel = texture2D(uTexture, vTexCoord);
    
    vec4 finalColour = texel; 
    
    if( uLightOn == 1 ){
    	
    	vec3 N = normalize(vEyespaceNormal.xyz);
    	vec3 E = normalize(vEyeVec); 
    
    	vec3 L = normalize(vLightDir);
    
    	// Reflect the vector. Use this or reflect(incidentV, N);
    	vec3 reflectV = reflect(-L, N);
  
	    vec4 ambient = texel * uAmbient;    	
    	vec4 diffuse = uDiffuse * max(dot(N, L), 0.0);
    	vec4 specular = uSpecular * pow(max(dot(reflectV, E), 0.0), uShinness);
    	
    	finalColour = ambient + diffuse + specular; 
    }
                           	
    gl_FragColor = finalColour * vec4(1,1,1,1);                       
}
