const int BONE_INFLUENCE_COUNT = 4;
const int MAX_BONE_COUNT = 30; 

precision highp float;
precision highp int; 

uniform mat4 uMVP;
uniform mat4 uNormalMatrix;
uniform highp mat4 uBoneTransforms[MAX_BONE_COUNT];
uniform int uBoneCount; 
uniform mat4 uBindShape;
uniform int uLightOn;
uniform vec4 uLightPosition; 

attribute highp vec4 aPosition;
attribute highp vec3 aNormal;  
attribute highp vec2 aTexCoord;
attribute highp vec4 aBoneIndex;
attribute highp vec4 aBoneWeight;

varying vec2 vTexCoord;
varying vec4 vNormal;
varying vec3 vEyespaceNormal;
varying vec3 vLightDir;
varying vec3 vEyeVec;

void main()                    
{                            
    vTexCoord = aTexCoord;
    
    mediump ivec4 indexOfBone = ivec4(aBoneIndex);
    highp vec4 weightOfBone = aBoneWeight; 
    
    highp vec4 vertex;
    mediump vec4 normal;
    
    if (uBoneCount > 0) {
        highp mat4 mat = uBoneTransforms[indexOfBone.x] * weightOfBone.x 
        	+ uBoneTransforms[indexOfBone.y] * weightOfBone.y; 
        		+ uBoneTransforms[indexOfBone.z] * weightOfBone.z
        			+ uBoneTransforms[indexOfBone.w] * weightOfBone.w;
                
        // resulting position after applying skinning
        vertex = mat * uBindShape * aPosition;
        normal = mat * uBindShape * vec4(aNormal, 1.0);
    } else {
        vertex = aPosition;
        normal = vec4(aNormal,1.0);        
    }   
    
    highp vec4 worldVertex = uMVP * vertex;
    
    vNormal = normal; 
    
    // normal
    vEyespaceNormal = vec3(uNormalMatrix * normal);        
    
    // light direction 
    vLightDir = normalize(uLightPosition.xyz - worldVertex.xyz); 
    vEyeVec = -worldVertex.xyz;	        
              
    gl_Position = worldVertex;    
}  