const int BONE_INFLUENCE_COUNT = 4;
const int MAX_BONE_COUNT = 72; 

uniform mat4 uMVP;
uniform mat4 uNormalMatrix;
uniform mat4 uBoneTransforms[MAX_BONE_COUNT];
uniform int uBoneCount; 

attribute vec4 aPosition;
attribute vec3 aNormal;  
attribute vec2 aTexCoord;
attribute vec4 aBoneIndex;
attribute vec4 aBoneWeight;

varying vec2 vTexCoord;
varying vec3 vNormal;
varying vec3 eyespaceNormal;

void main()                    
{                            
    vTexCoord = aTexCoord;
    vNormal = aNormal;	
    
    vec4 skinnedPosition = aPosition;     

	if( uBoneCount > 0 ){          
	    
	    vec4 p0 = uBoneTransforms[int(aBoneIndex.x)] * aPosition;
    	vec4 p1 = uBoneTransforms[int(aBoneIndex.y)] * aPosition;
    	vec4 p2 = uBoneTransforms[int(aBoneIndex.z)] * aPosition;
    	vec4 p3 = uBoneTransforms[int(aBoneIndex.w)] * aPosition;
    	skinnedPosition = p0 * aBoneWeight.x + p1 * aBoneWeight.y + p2 * aBoneWeight.z + p3 * aBoneWeight.w;    
    
    	skinnedPosition.w = 1.0;
    
 	} 
    
    
    // normal
	eyespaceNormal = vec3(uNormalMatrix * vec4(aNormal, 1.0));
    	  	  
    gl_Position = uMVP * skinnedPosition;    
}  