/**
 * AnimationClip.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 29, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.animations;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import com.wmp.nofrills3d.geometry.Bone;

public class AnimationClip {		
	
	protected List<Keyframe> keyframes;	
		
	public int[] boneKeyframesCount = null;
	
	public int[][] boneKeyframes = null; 
	
	protected float duration = 0.0f;
	
	protected float[] frameTimes = null; 
	
	public AnimationClip(){
		keyframes = new Vector<Keyframe>(); 
	}	
	
	public void init(Bone rootBone){
//		chainKeyframes(); 
		buildBoneKeyframeMatrix(rootBone); 
	}
	
	// replaced with bone matrix 
//	protected void chainKeyframes(){
//		for( int i=0; i<keyframes.size(); i++){
//			Keyframe currentKeyframe = keyframes.get(i);
//			
//			int j = i+1; 
//			for( int k=0; k<keyframes.size(); k++ ){
//				j++; 
//				
//				if( j >= keyframes.size() ){
//					j = 0; 
//				}
//				
//				Keyframe nextKeyframe = keyframes.get(i);
//				
//				if( currentKeyframe.bone.index == nextKeyframe.bone.index ){
//					if( nextKeyframe.previousKeyframe != null ){
//						currentKeyframe.nextKeyframe = nextKeyframe; 
//						nextKeyframe.previousKeyframe = currentKeyframe; 
//						
//						if( j < i ){
//							currentKeyframe.isLastKeyframe = true; 
//						}
//					}
//					break; 
//				}
//			}
//		}
//	}
	
	protected void buildBoneKeyframeMatrix(Bone rootBone){
		int boneCount = rootBone.size(); 
		boneKeyframesCount = new int[boneCount];
		
		for( int i=0; i<boneCount; i++ ){
			boneKeyframesCount[i] = 0; 
		}
		
		for( Keyframe keyframe : keyframes ){
			boneKeyframesCount[keyframe.bone.index]++; 
		}
		
		// create bone keyframe index 
		boneKeyframes = new int[boneCount][];
		
		for( int boneIndex = 0; boneIndex<boneCount; boneIndex++ ){
			int currentIndex = 0;
			boneKeyframes[boneIndex] = new int[boneKeyframesCount[boneIndex]];
			
			for( int i=0; i<keyframes.size(); i++ ){
				Keyframe currentKeyframe = keyframes.get(i);
				if( boneIndex == currentKeyframe.bone.index ){
					boneKeyframes[boneIndex][currentIndex++] = i; 
				}
			}
		}
		
	}
	
	public void addKeyframe(Keyframe keyframe){
		this.keyframes.add(keyframe);
		updateDuration();
	}
	
	public Keyframe getKeyframeAtIndex(int index){
		return keyframes.get(index);
	}
	
	public Keyframe getKeyframeAtTime(float time){
		return null; 
	}	
	
	public float getDuration(){
		return duration; 
	}
	
	protected void updateDuration(){
		float maxTime = 0.0f; 
		for( Keyframe kf : keyframes ){
			if( kf.time > maxTime ){
				maxTime = kf.time; 
			}
		}
		
		duration = maxTime;
		
	}
	
	protected void buildFrameTimes(){
		List<Float> times = new Vector<Float>(); 
		Float currentTime = 0.0f; 
		
		for( int i=0; i<keyframes.size(); i++ ){
			Keyframe frame = keyframes.get(i);
			if( currentTime.floatValue() != frame.time ){
				currentTime = Float.valueOf(frame.time); 
				times.add(currentTime);
			}
		}
		
		frameTimes = new float[times.size()];
		for( int i=0; i<times.size(); i++ ){
			frameTimes[i] = times.get(i).floatValue();
		}
	}
	
	public float getTimeForFrame(int frame){
		if( frameTimes == null ){
			buildFrameTimes(); 
		}
		
		if( frame < 0 || frame >= frameTimes.length ){
			return -1; 
		}
		
		return frameTimes[frame];
	}
	
	public void sortKeyframes(){
		Collections.sort(keyframes);
	}
}
