/**
 * NoFrills3D
 * com.wmp.nofrills3d.effects
 * 
 * @author Joshua Newnham
 * @copyright We Make Play Ltd. 2013 
 * 
 */
package com.wmp.nofrills3d.effects;

import java.util.ArrayList;
import java.util.List;

import com.wmp.nofrills3d.material.Texture;

/**
 * 
 */
public class EffectsProcessor {
	
	protected RenderTarget renderTarget;
	
	protected List<Effect> effects; 
	
	protected int effectsCount = 0; 
	
	public EffectsProcessor(RenderTarget renderTarget){
		effects = new ArrayList<Effect>(); 
		
		if( renderTarget != null ){
			this.renderTarget = new RenderTarget(0, 0, 0, null);
		} else{
			this.renderTarget = renderTarget;  
		}
	}
	
	public void addEffect(Effect effect){
		this.effects.add(effect);	
		effectsCount++;  
	}
	
	public Texture process(Texture input){
		if( effects.size() == 0 ){
			return null; 
		}
		
		Texture output = null; 
		
		for( int i=0; i<effects.size(); i++ ){
			output = effects.get(i).getOutput();
			renderTarget.bind(output);
			effects.get(i).process(input);
			
			input = output; 
		}
		
		return output; 
	}
	
	public void dispose(){
		
		if( renderTarget != null ){
			renderTarget.dispose(); 
			renderTarget = null; 
		}
		
		for(Effect effect : effects){
			effect.dispose(); 
		}
		
		effects.clear(); 
		
		effectsCount = 0; 
	}
	
	public boolean hasEffects(){
		return effectsCount > 0; 
	}
	
}
