/**
 * NoFrills3D
 * com.wmp.nofrills3d.effects
 * 
 * @author Joshua Newnham
 * @copyright We Make Play Ltd. 2013 
 * 
 */
package com.wmp.nofrills3d.effects;

import android.opengl.GLES20;

import com.wmp.nofrills3d.material.Texture;
import com.wmp.nofrills3d.utils.ContentLoader;

/**
 * 
 */
public class RenderTarget {

	public static final int NONE							= 0x0000; 
	public static final int RENDER_TO_TEXTURE 				= 0x0001; 
	public static final int RENDER_TO_DEPTH_RENDER_BUFFER  	= 0x0010; 
	
	protected int flags = -1; 
	
	protected int width = 0; 
	protected int height = 0; 
	
	protected int frameBuffer = -1; 
	protected Texture texture = null; 
	protected int depthBufferId = -1; 
	
	public RenderTarget(int flags, int width, int height, Texture texture){
		this.flags = flags; 
		this.width = width; 
		this.height = height; 
		
		this.texture = texture; 
		
		init(); 
	}
	
	protected void init(){
		// create frame buffer 
		int[] fb = new int[1];
    	
    	GLES20.glGenFramebuffers(1, fb, 0);
    	
    	frameBuffer = fb[0];    	    	
    	
    	if( (this.flags & RENDER_TO_TEXTURE) > 0 ){
    		if( texture == null ){
    			texture = ContentLoader.createTexture(width, height);
    		}    		
    	}
    	
    	if( (this.flags & RENDER_TO_DEPTH_RENDER_BUFFER) > 0 ){
    		int[] depthRb = new int[1]; 
    		GLES20.glGenRenderbuffers(1, depthRb, 0);
    		depthBufferId = depthRb[0]; 
    		
    		GLES20.glBindRenderbuffer(GLES20.GL_RENDERBUFFER, depthRb[0]);
    		GLES20.glRenderbufferStorage(GLES20.GL_RENDERBUFFER, GLES20.GL_DEPTH_COMPONENT16, width, height);
    	}
	}
	
	public void bind(){
		this.bind(this.texture); 
	}
	
	public void bind(Texture texture){
		GLES20.glViewport(0, 0, width, height);
		
		// bind our offscreen framebuffer
    	GLES20.glBindFramebuffer(GLES20.GL_FRAMEBUFFER, frameBuffer);
		
    	if( texture != null ){
    		// specify texture as color attachment
    		GLES20.glFramebufferTexture2D(GLES20.GL_FRAMEBUFFER, GLES20.GL_COLOR_ATTACHMENT0, GLES20.GL_TEXTURE_2D, texture.getTextureId(), 0);
    	}
    	
    	if( depthBufferId > 0 ){
    		// attach render buffer as depth buffer    	
    		GLES20.glFramebufferRenderbuffer(GLES20.GL_FRAMEBUFFER, GLES20.GL_DEPTH_ATTACHMENT, GLES20.GL_RENDERBUFFER, depthBufferId);
    	}
	}
	
	public void unbind(){
		GLES20.glBindFramebuffer(GLES20.GL_FRAMEBUFFER, 0);
	}
	
	public Texture getTexture(){
		return texture; 
	}
	
	public void dispose(){
		if( texture != null ){
			texture.dispose(); 
			texture = null; 
		}
		
		if( depthBufferId != -1 ){
			GLES20.glDeleteRenderbuffers(1, new int[] { depthBufferId }, 0);						
			depthBufferId = -1; 
		}
		
		if( frameBuffer != -1 ){
			GLES20.glDeleteFramebuffers(1, new int[] { frameBuffer }, 0);
			frameBuffer = -1; 
		}
	}
}
