/**
 * NoFrills3D
 * com.wmp.nofrills3d.effects
 * 
 * @author Joshua Newnham
 * @copyright We Make Play Ltd. 2013 
 * 
 */
package com.wmp.nofrills3d.effects;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import android.opengl.GLES20;

import com.wmp.nofrills3d.material.Shader;
import com.wmp.nofrills3d.material.Texture;
import com.wmp.nofrills3d.scene.Constants;

/**
 * 
 */
public class TextureRenderer {

	protected static final int VERT_COMPONENT_COUNT = 2;

	protected float _verticesData[] = { // in counterclockwise order:
		-1.0f, -1.0f, 1.0f, -1.0f, 1.0f, 1.0f, -1.0f, 1.0f };

	protected static final int TEX_COORDS_COMPONENT_COUNT = 2;

	protected float _texCoordData[] = { // in counterclockwise order:
			0.0f, 0.0f, // bottom right
			1.0f, 0.0f, // top right
			1.0f, 1.0f, // bottom left
			0.0f, 1.0f // top left
	};

	protected short _indiciesData[] = { 0, 1, 2, 0, 2, 3 }; 

	private FloatBuffer _textureVertices;
	private FloatBuffer _positionVertices;
	private ShortBuffer _triangles;
	
	protected int viewWidth; 
	protected int viewHeight; 
	
	public TextureRenderer(int viewWidth, int viewHeight){
		this.viewWidth = viewWidth; 
		this.viewHeight = viewHeight;
		
		//scaleVerts(); 
		initBuffers(); 
	}
	
	protected void scaleVerts(){
		float w = (float)viewWidth/2.0f; 
		float h = (float)viewHeight/2.0f; 
		
		_verticesData[0] = -w; // left 
    	_verticesData[1] = -h; // top 
    	_verticesData[2] = w; // right 
    	_verticesData[3] = -h; // top 
    	_verticesData[4] = w; 
    	_verticesData[5] = h; 
    	_verticesData[6] = -w; 
    	_verticesData[7] = h;                 
	}
	
	protected void initBuffers(){
    	
    	ByteBuffer bb = ByteBuffer.allocateDirect(_verticesData.length * 4);
        bb.order(ByteOrder.nativeOrder());
        _positionVertices = bb.asFloatBuffer();
        _positionVertices.put(_verticesData);
        _positionVertices.position(0);

        ByteBuffer dlb = ByteBuffer.allocateDirect(_indiciesData.length * 2);
        dlb.order(ByteOrder.nativeOrder());
        _triangles = dlb.asShortBuffer();
        _triangles.put(_indiciesData);
        _triangles.position(0);

        ByteBuffer bb2 = ByteBuffer.allocateDirect(_texCoordData.length * 4);
        bb2.order(ByteOrder.nativeOrder());
        _textureVertices = bb2.asFloatBuffer();
        _textureVertices.put(_texCoordData);
        _textureVertices.position(0);        
    }
	
	protected void updatePositionBuffers(){
		_positionVertices.position(0); 
		_positionVertices.put(_verticesData);
        _positionVertices.position(0);
	}
	
	public void render(Shader shader, Texture texture){
		GLES20.glDisable(GLES20.GL_DEPTH_TEST);		
		GLES20.glDisable(GLES20.GL_CULL_FACE);
        GLES20.glDisable(GLES20.GL_BLEND);
		
		GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT); 
		
		shader.bind();
		
		bind(shader, texture); 
		
		// Set viewport
        GLES20.glViewport(0, 0, viewWidth, viewHeight);                               

        // Draw
        GLES20.glClearColor(0.3f, 0.3f, 0.3f, 1.0f);
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
        GLES20.glDrawElements(GLES20.GL_TRIANGLES, _indiciesData.length, GLES20.GL_UNSIGNED_SHORT, _triangles);        		
        
        unbind(shader);
    }
    
	protected void bind(Shader shader, Texture texture){    	    	
    	// Set the vertex attributes                
        GLES20.glVertexAttribPointer(shader.getAttribLocation(Constants.ATTRIBUTE_POSITION), 2, GLES20.GL_FLOAT, false,0, _positionVertices); 
        GLES20.glEnableVertexAttribArray(shader.getAttribLocation(Constants.ATTRIBUTE_POSITION));
        
        GLES20.glVertexAttribPointer(shader.getAttribLocation(Constants.ATTRIBUTE_TEX_COORD), 2, GLES20.GL_FLOAT, false,0, _textureVertices);
        GLES20.glEnableVertexAttribArray(shader.getAttribLocation(Constants.ATTRIBUTE_TEX_COORD));
        
        // Set the input texture
        GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, texture.getTextureId());
        GLES20.glUniform1i( shader.getUniformLocation(Constants.UNIFORM_TEXTURE_0), 0);
    }
    
    protected void unbind(Shader shader){
    	
    	GLES20.glDisableVertexAttribArray(shader.getAttribLocation(Constants.ATTRIBUTE_TEX_COORD));
    	GLES20.glDisableVertexAttribArray(shader.getAttribLocation(Constants.ATTRIBUTE_POSITION));
    	
    }
	
}
