/**
 * Transform.java (ColourExplosion)
 * Created by Joshua Newnham on Apr 9, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities;

import java.util.Iterator;

import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.physics.ColliderCollection;
import com.wmp.nofrills3d.physics.ContactCollection;
import com.wmp.nofrills3d.physics.ICollider;
import com.wmp.nofrills3d.physics.IColliderCollection;
import com.wmp.nofrills3d.scene.IScene;

public class Entity extends Transform implements IColliderCollection {			
	
	private boolean loaded = false; 
	
	protected IScene scene; 
	
	protected ColliderCollection colliderCollection = new ColliderCollection(); 
	
	public Entity(){
				
	}
	
	public void load(IScene scene){
		this.scene = scene; 
		loaded = true; 
	}
	
	public void dispose(){
		this.unload(true);
	}
		
	public void unload(boolean fullDispose){
		loaded = false; 
	}
	
	public boolean isLoaded(){
		return loaded; 
	}
	
	public void update(float elapsedTime){
		super.update(elapsedTime);
	}
	
	public void postUpdate(float elapsedTime){
		super.postUpdate(elapsedTime);
		
		colliderCollection.update(elapsedTime);
	}
	
	public void render(Matrix4 projectionMatrix, Matrix4 viewTransform){
		render(projectionMatrix, viewTransform, 0);
	}
	
	public void render(Matrix4 projectionMatrix, Matrix4 viewTransform, int pass){
		
	}	
	
	public void onHandleCollisions(ContactCollection contacts){
		
	}
	
	public boolean isCollidable(){
		return colliderCollection.size() > 0; 
	}

	@Override
	public void addCollider(ICollider collider) {
		colliderCollection.addCollider(collider);
	}

	@Override
	public ICollider getCollidableAtIndex(int index) {
		return colliderCollection.getCollidableAtIndex(index);
	}

	@Override
	public ICollider getCollidableWithTag(String tag) {
		return colliderCollection.getCollidableWithTag(tag);
	}
	
	public Iterator<ICollider> getColliderCollectionIterator(){
		return colliderCollection.iterator(); 
	}	

	@Override
	public int collidesWith(ICollider other, ContactCollection contacts) {
		return colliderCollection.collidesWith(other, contacts);
	}	
	
	public int collidesWith(Entity other, ContactCollection contacts) {
		int sum = 0; 
		
		for( ICollider collider : colliderCollection ){
			for( ICollider otherCollider : other.colliderCollection ){
				sum += collider.collidesWith(otherCollider, contacts);
			}
		}
		return sum; 
	}
}
