/**
 * Model.java (ColourExplosion)
 * Created by Joshua Newnham on Apr 9, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities;

import com.wmp.nofrills3d.geometry.IMesh;
import com.wmp.nofrills3d.material.Material;
import com.wmp.nofrills3d.material.Shader;
import com.wmp.nofrills3d.material.ShadowMaterial;
import com.wmp.nofrills3d.material.Texture;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.scene.IScene;

public class Model extends Entity{

	public Material material; 
	
	public IMesh mesh; 
	
	public Texture texture;
	
	protected boolean initilised = false; 
	
	public Model(){
		
	}
	
	public Model(IMesh mesh, Material material){
		this(mesh, material, null);
	}
	
	public Model(IMesh mesh, Material material, Texture texture){
		initModel(mesh, material, texture); 
	}
	
	protected void initModel(IMesh mesh, Material material, Texture texture){
		this.mesh = mesh; 
		this.material = material; 
		this.texture = texture; 
		
		this.tag = mesh.getTag();
		
		if( this.mesh.getBoundingBox() != null ){
			this.mesh.getBoundingBox().setTransform(this);
		}
		
		initilised = true; 
	}
	
	@Override
	public void update(float elapsedTime){
		if( !enabled ) return; 
		
		// refreshes model matrix 
		super.update(elapsedTime);
	}
	
	@Override
	public void render(Matrix4 projectionMatrix, Matrix4 viewTransform, int pass){
		if( !canRender() ) return; 				
		
		// bind material 
		material.modelTransform = this.modelTransform; 
		material.viewTransform = viewTransform; 
		material.projectionTransform = projectionMatrix; 
		material.texture = texture; 
		material.pass = pass; 
		
		material.bind(); 
		mesh.bind(material); 
		
		mesh.draw(); 
		
		mesh.unbind(material); 
		material.unbind(); 		
	}
	
	protected boolean canRender(){
		if( !visible ) return false;
		
		if( mesh == null || material == null ) return false;
		
		return true; 
	}
	
	@Override
	public void load(IScene scene){
		super.load(scene);
	}
	
	@Override
	public void unload(boolean fullDispose){
		
		if( fullDispose ){
			if( texture != null ){
				texture.dispose(); 
				texture = null; 
			}
			
			if( mesh != null ){
				mesh.dispose();
				mesh = null; 
			}
			
			if( material != null ){
				material.dispose(); 
				material = null; 
			}
		}
		
		super.unload(fullDispose); 
	}
	
}
