package com.wmp.nofrills3d.entities;

import android.opengl.Matrix;

import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.math.Vector3;

public class Transform {

	public String tag = "";
	
	protected Vector3 position = Vector3.createZero();
	
	protected Vector3 rotation = Vector3.createZero();
	
	protected Vector3 scale = Vector3.createOne();
	
	public Matrix4 modelTransform = Matrix4.createIdentity();
	
	public Matrix4 invModelTransform = Matrix4.createIdentity();
	
	public boolean enabled = true; 
	
	public boolean visible = true;
	
	public Transform(){
		
	}
	
	public void update(float elapsedTime){
		refreshModelMatrix(); 
	}
	
	public void postUpdate(float elapsedTime){
		
	}
	
	public void refreshModelMatrix(){
		modelTransform.reset(); 
				
		Matrix.translateM(modelTransform.matrix, 0, position.x, position.y, position.z);
		Matrix.rotateM(modelTransform.matrix, 0, rotation.x, 1.0f, 0.0f, 0.0f);
		Matrix.rotateM(modelTransform.matrix, 0, rotation.y, 0.0f, 1.0f, 0.0f);
		Matrix.rotateM(modelTransform.matrix, 0, rotation.z, 0.0f, 0.0f, 1.0f);
		Matrix.scaleM(modelTransform.matrix, 0, scale.x, scale.y, scale.z);
		
		invModelTransform.reset();
		Matrix4.invert(modelTransform, invModelTransform);
	}
	
	public Vector3 getForward(){
		return modelTransform.getForward().normalise(); 
	}
	
	public Vector3 getRight(){
		return modelTransform.getRight().normalise(); 
	}
	
	public Vector3 getUp(){
		return modelTransform.getUp().normalise(); 
	}
	
	public String getTag() {
		return tag; 
	}
	
	public Vector3 getPosition(){
		return position; 
	}
	
	public Vector3 getRotation(){
		return rotation; 
	}
	
	public Vector3 getScale(){
		return scale; 
	}
	
	public void setPosition(float x, float y, float z){
		position.x = x; position.y = y; position.z = z; 
	}
	
	public void setPosition(Vector3 position){
		this.setPosition(position.x, position.y, position.z);  
	}
	
	public void move(float dx, float dy, float dz){
		this.position.x += dx; 
		this.position.y += dy; 
		this.position.z += dz; 
	}
	
	public void setRotation(float x, float y, float z){
		rotation.x = x; rotation.y = y; rotation.z = z; 
	}
	
	public void setRotation(Vector3 rotation){
		this.setPosition(rotation.x, rotation.y, rotation.z);  
	}
	
	public void rotate(float x, float y, float z){
		rotation.x += x; 
		rotation.y += y; 
		rotation.z += z; 
	}
	
	public void setScale(float sx, float sy, float sz){
		this.scale.x = sx;
		this.scale.y = sy; 
		this.scale.z = sz; 
	}
	
	public void setScale(float s){
		this.scale.x = this.scale.y = this.scale.z = s;   
	}
	
	public Vector3 transformFromWorldToModelSpace(Vector3 target){ 
		Matrix4 worldMat= Matrix4.createTranslation(target.x, target.y, target.z);
		Matrix4 modelMat = Matrix4.createIdentity(); 
		
		Matrix4.multiply(invModelTransform, worldMat, modelMat);
		
		Matrix4.decomposeTranslation(modelMat, target);
		
		return target; 
	}
	
	public Vector3 transformFromModelToWorldSpace(Vector3 target){ 
		Matrix4 worldMat= Matrix4.createTranslation(target.x, target.y, target.z);
		Matrix4 modelMat = Matrix4.createIdentity(); 
		
		Matrix4.multiply(modelTransform, worldMat, modelMat);
		
		Matrix4.decomposeTranslation(modelMat, target);
		
		return target; 
	}
}
