/**
 * ModelLoader.java (WMP3D)
 * Created by Joshua Newnham on Apr 10, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities.loaders;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import android.content.Context;
import android.os.Handler;

import com.wmp.colourexplosion.R;
import com.wmp.nofrills3d.geometry.IMesh;
import com.wmp.nofrills3d.material.Material;

public abstract class ModelLoader implements Runnable{	
	
	protected static ExecutorService executorService = null;
	
	public String tag = "";
	
	protected Context context;
	
	protected ModelLoaderDelegate delegate; 
	
	protected boolean resourcesLoaded = false; 	
	
	protected int loadState = 0; 
	
	protected Handler handler = null; 
	
	protected int[] resId = null; 
	
	protected static synchronized ExecutorService getSharedExecutorService(){
		if( executorService == null ){
			executorService = Executors.newFixedThreadPool(1);
		}
		
		return executorService; 
	}
	
	public ModelLoader(Context context){
		this.context = context;
		this.handler = new Handler();
	}	
	
	public void parseAsync(ModelLoaderDelegate delegate, int... resId){
		this.delegate = delegate;
		
		this.resId = resId; 				
		
		getSharedExecutorService().execute(this);
	}
	
	public void parse(int... resId){		
		this.resId = resId; 
		parseResource(resId);			 				
	}
	
	public void run(){
		parseResource(resId);
		
		final ModelLoader instance = this; 
		
		if( delegate != null ){
			handler.post(new Runnable() {								
				@Override
				public void run() {
					delegate.onModelResourcesParsed(instance, loadState);
				}
			});			
		}
	}
	
	protected abstract void parseResource(int... resId);
	
	public abstract IMesh[] buildAndFetchGeometry();
	
	protected abstract IMesh[] getParsedMeshes(); 
	
	public Context getContext(){
		return context; 
	}
	
	public Material getDefaultMaterial(){		
		return new Material(context, R.raw.phong_vs, R.raw.phong_fs);
	}
	
	protected float[] packVertexData(Float[] vertexData1, int vertexData1Count, Float[] vertexData2, int vertexData2Count, Float[] vertexData3, int vertexData3Count){
		int size = (vertexData1 != null ? vertexData1.length : 0) + (vertexData2 != null ? vertexData2.length : 0) + (vertexData3 != null ? vertexData3.length : 0); 
		
		if( size == 0 ){
			return null; 
		}
		
		float packedData[] = new float[size];
		
		int index1 = 0; 
		int index2 = 0; 
		int index3 = 0; 
		
		for( int i=0; i<size; i++ ){
			if( vertexData1 != null ){
				for( int j=0; j<vertexData1Count; j++ ){
					packedData[i++] = vertexData1[index1++];
					packedData[i++] = vertexData1[index1++];
					packedData[i++] = vertexData1[index1++];
				}
			}
			
			if( vertexData2 != null ){
				for( int j=0; j<vertexData2Count; j++ ){
					packedData[i++] = vertexData2[index2++];
					packedData[i++] = vertexData2[index2++];
					packedData[i++] = vertexData2[index2++];
				}
			}
			
			if( vertexData3 != null ){
				for( int j=0; j<vertexData3Count; j++ ){
					packedData[i++] = vertexData3[index3++];
					packedData[i++] = vertexData3[index3++];
					packedData[i++] = vertexData3[index3++];
				}
			}
		}
		
		return packedData; 
	}		
}
