/**
 * CAnimationParser.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 29, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities.loaders.collada;

import java.io.IOException;
import java.util.List;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import com.wmp.nofrills3d.entities.loaders.collada.CControllerParser.ParserStateEnum;
import com.wmp.nofrills3d.entities.loaders.collada.COLLADAModelLoader.CDataTypeEnum;
import com.wmp.nofrills3d.utils.Logger;

public class CAnimationParser extends CBaseParser {

	protected XmlPullParser xpp = null;
	protected List<CAnimation> animationLibrary = null; 
	protected CAnimation currentAnimation = null;
	protected CSource currentSource = null; 
	
	public static CAnimationParser parse(XmlPullParser xpp, List<CAnimation> animationLibrary){
		return new CAnimationParser(xpp, animationLibrary).start(); 
	}
	
	private CAnimationParser(XmlPullParser xpp, List<CAnimation> animationLibrary){
		this.xpp = xpp; 
		this.animationLibrary = animationLibrary; 
	}		
	
	public CAnimationParser start(){
		if( !xpp.getName().equals(CConstants.TAG_LIBRARY_ANIMATIONS) ){
			return this; 
		}
		
		try{
			while(true){
				xpp.next(); 
				int eventType = xpp.getEventType();
				
				if (eventType == XmlPullParser.END_TAG) {
					if( xpp.getName().equals(CConstants.TAG_LIBRARY_ANIMATIONS) ){
						break; 
					} else{
						parseEndTag(xpp);
					}
				} 
				else if( eventType == XmlPullParser.START_TAG ){
					parseStartTag(xpp);
				}
				else if( eventType == XmlPullParser.TEXT ){
					parseText(xpp);
				}
			}
		
		} catch( XmlPullParserException e ){
			
		} catch( IOException e ){
			
		}
		
		return this; 
	} 
	
	/* (non-Javadoc)
	 * @see com.wmp.nofrills3d.entities.loaders.collada.CBaseParser#parseStartTag(org.xmlpull.v1.XmlPullParser)
	 */
	@Override
	protected void parseStartTag(XmlPullParser xpp) throws XmlPullParserException, IOException {
		String tag = xpp.getName(); 
		
		if( tag.equals(CConstants.TAG_ANIMATION)){
			currentAnimation = new CAnimation();
			
			for( int i=0; i<xpp.getAttributeCount(); i++ ){
				String attr = xpp.getAttributeName(i);
				String val = xpp.getAttributeValue(i);
				
				if( attr.equals(CConstants.ATTR_ID)){
					currentAnimation.id = val; 
				}				
			}			
		} else if( tag.equals(CConstants.TAG_SOURCE)){
			currentSource = new CSource(); 
			
			for( int i=0; i<xpp.getAttributeCount(); i++ ){
				String attr = xpp.getAttributeName(i);
				String val = xpp.getAttributeValue(i);
				
				if( attr.equals(CConstants.ATTR_ID)){
					currentSource.id = val; 
				}				
			}
		} else if( tag.equals(CConstants.TAG_SOURCE_FLOAT_ARRAY )){
			if( currentSource != null ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_ID)){
						 // ignore 
					} else if( attr.equals(CConstants.ATTR_COUNT)){
						 currentSource.length = Integer.parseInt(val);
						 currentSource.dataAsFloat = new float[currentSource.length];
						 currentSource.dataType = CDataTypeEnum.Float;
					}
				}
				
				xpp.next(); 
				
				parseDelimitedFloatArrayForSource(currentSource, xpp.getText());
			}						
		} else if( tag.equals(CConstants.TAG_SOURCE_NAME_ARRAY)){
			if( currentSource != null ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_ID)){
						 // ignore 
					} else if( attr.equals(CConstants.ATTR_COUNT)){
						 currentSource.length = Integer.parseInt(val);
						 currentSource.dataAsString = new String[currentSource.length];
						 currentSource.dataType = CDataTypeEnum.Name;
					}
				}
				
				xpp.next(); 
				
				parseDelimitedStringArrayForSource(currentSource, xpp.getText());
			}	
		} else if( tag.equals(CConstants.TAG_SOURCE_ACCESSOR)){
			if( currentSource != null ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_COUNT)){
						currentSource.count = Integer.parseInt(val); 
					} else if( attr.equals(CConstants.ATTR_STRIDE)){
						currentSource.stride = Integer.parseInt(val);
						currentSource.params = new CSourceParam[1];
					}				
				}
			}					
		} else if( tag.equals(CConstants.TAG_SOURCE_ACCESSOR_PARAM)){
			if( currentSource != null ){				
				CSourceParam param = new CSourceParam(); 
				
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_NAME)){
						param.name = val;  
					} else if( attr.equals(CConstants.ATTR_TYPE)){
						if( val.equals(CConstants.VALUE_PARAM_FLOAT4x4) ){
							param.type = CDataTypeEnum.Float4x4; 
						} else if( val.equals(CConstants.VALUE_PARAM_FLOAT) ){
							param.type = CDataTypeEnum.Float; 
						} else if( val.equals(CConstants.VALUE_PARAM_NAME) ){
							param.type = CDataTypeEnum.Name; 
						}
					}				
				}
				
				currentSource.params[currentSource.nextParamIndex] = param;
				currentSource.nextParamIndex++;
				
				if( param.name.equals(CConstants.VALUE_PARAM_NAME_TIME)){
					currentAnimation.timeSource = currentSource; 
				} else if( param.name.equals(CConstants.VALUE_PARAM_NAME_TRANSFORM)){
					currentAnimation.transformSource = currentSource; 
				}
				
				if( currentSource.nextParamIndex >= currentSource.params.length ){
					CSourceParam[] newParamsArray = new CSourceParam[currentSource.params.length+1];
					System.arraycopy(currentSource.params, 0, newParamsArray, 0, currentSource.params.length);
					currentSource.params = newParamsArray; 
				}
			}					
		} else if( tag.equals(CConstants.TAG_INPUT) ){
			if( currentAnimation != null ){
				
				String semantic = null; 
				String source = null; 
				
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_SEMANTIC)){
						semantic = val; 
					} else if( attr.equals(CConstants.ATTR_SOURCE)){
						source = val.replace("#", ""); 
					}				
				}
				
				if( semantic != null && source != null ){
					CSource cSource = currentAnimation.findInputWithSourceId(source);
					
					if( cSource != null ){
						cSource.type = parseStringToSemantic(semantic);
					} else{
						Logger.LogWarning(TAG, "Unable to find source with ID " + source);						
					}
				}								
			}
		} else if( tag.equals(CConstants.TAG_CHANNEL) ){
			if( currentAnimation != null ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.TAG_TARGET)){
						currentAnimation.target = val; 
						String[] boneChannelPair = val.split("/");
						if( boneChannelPair != null && boneChannelPair.length == 2 ){
							currentAnimation.bone = boneChannelPair[0]; 
							currentAnimation.channel = boneChannelPair[1]; 
						} else{
							Logger.LogWarning(TAG, "Unable to split channel " + val);
						}
					} 			
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see com.wmp.nofrills3d.entities.loaders.collada.CBaseParser#parseText(org.xmlpull.v1.XmlPullParser)
	 */
	@Override
	protected void parseText(XmlPullParser xpp) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.wmp.nofrills3d.entities.loaders.collada.CBaseParser#parseEndTag(org.xmlpull.v1.XmlPullParser)
	 */
	@Override
	protected void parseEndTag(XmlPullParser xpp) {
		String tag = xpp.getName();
		
		if( tag.equals(CConstants.TAG_ANIMATION )){
			if( currentAnimation != null ){
				animationLibrary.add(currentAnimation);
				currentAnimation = null; 
			}
		} else if( tag.equals(CConstants.TAG_SOURCE )){						
			if( currentAnimation != null && currentSource != null ){
				if( currentAnimation != null ){
					currentAnimation.sources.add(currentSource);
				}
				
				currentSource = null; 
			}
		} 

	}

}
