/**
 * CBaseParser.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 23, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities.loaders.collada;

import java.io.IOException;
import java.util.StringTokenizer;
import java.util.concurrent.Semaphore;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import com.wmp.nofrills3d.entities.loaders.collada.COLLADAModelLoader.CSourceSemantic;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.utils.Logger;

public abstract class CBaseParser {
	
	public static final String TAG = "CBaseParser";

	protected abstract void parseStartTag(XmlPullParser xpp) throws XmlPullParserException, IOException;
	
	protected abstract void parseText(XmlPullParser xpp);
	
	protected abstract void parseEndTag(XmlPullParser xpp);
	
	protected void parseDelimitedFloatArrayForSource(CSource source, String data){
		StringTokenizer parts = new StringTokenizer(data, " ");
		int numTokens = parts.countTokens();
		
		if (numTokens == 0)
			return;
		
		for( int i=0; i<numTokens; i++ ){
			String val = parts.nextToken();
			
			if( val == null ){
				source.dataAsFloat[i] = 0.0f; 
				continue; 
			}
			
			source.dataAsFloat[i] = Float.parseFloat(val.trim());			
		}		
	}
	
	protected void parseDelimitedStringArrayForSource(CSource source, String data){
		StringTokenizer parts = new StringTokenizer(data, " ");
		int numTokens = parts.countTokens();
		
		if (numTokens == 0)
			return;
		
		for( int i=0; i<numTokens; i++ ){
			String val = parts.nextToken();
			
			if( val == null ){
				source.dataAsString[i] = ""; 
				continue; 
			}
			
			source.dataAsString[i] = val.trim();			
		}		
	}
	
	protected void parseDelimitedMatrix(Matrix4 matrix, String data){
		StringTokenizer parts = new StringTokenizer(data, " ");
		int numTokens = parts.countTokens();
		
		if (numTokens == 0)
			return;
				
		if( numTokens != 16 ){
			Logger.LogError(TAG, "Trying to parse a matrix that doesn't have 16 elements");
			return;  
		}
		
		for( int i=0; i<numTokens; i++ ){
			String val = parts.nextToken();
			matrix.matrix[i] = Float.parseFloat(val);			
		}
	}
	
	protected CSourceSemantic parseStringToSemantic(String source){
		if( source.equals(CConstants.VALUE_SEMANTIC_VERTEX)){
			return CSourceSemantic.Vertex;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_JOINT)){
			return CSourceSemantic.Joint;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_NORMAL)){
			return CSourceSemantic.Normal;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_TEXCOORD)){
			return CSourceSemantic.TexCoord;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_WEIGHT)){
			return CSourceSemantic.Weight;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_INV_BIND_MATRIX)){
			return CSourceSemantic.InvBindMatrix;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_INPUT)){
			return CSourceSemantic.Input;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_OUTPUT)){
			return CSourceSemantic.Output;
		} else if( source.equals(CConstants.VALUE_SEMANTIC_INTERPOLATION)){
			return CSourceSemantic.Interpolation;
		}
		
		return CSourceSemantic.Undefined; 
	}
}
