/**
 * GeometriesParser.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 16, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities.loaders.collada;

import java.io.IOException;
import java.util.List;
import java.util.StringTokenizer;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import com.wmp.nofrills3d.entities.loaders.collada.COLLADAModelLoader.CDataTypeEnum;
import com.wmp.nofrills3d.entities.loaders.collada.COLLADAModelLoader.CSourceSemantic;

public class CGeometriesParser extends CBaseParser {
		
	protected XmlPullParser xpp = null; 
	protected List<CGeometry> geometryLibrary; 
	
	protected CGeometry currentGeometry = null; 
	protected CSource currentSource = null; 
	protected CPolyList currentPolyList; 
	
	public static CGeometriesParser parse(XmlPullParser xpp, List<CGeometry> geometryLibrary){
		return new CGeometriesParser(xpp, geometryLibrary).start(); 
	}
	
	private CGeometriesParser(XmlPullParser xpp, List<CGeometry> geometryLibrary){
		this.xpp = xpp; 
		this.geometryLibrary = geometryLibrary; 
	}
	
	public CGeometriesParser start(){
		if( !xpp.getName().equals(CConstants.TAG_LIBRARY_GEOMETRIES) ){
			return this; 
		}
		
		try{
		
			while(true){
				xpp.next(); 
				int eventType = xpp.getEventType();
				
				if (eventType == XmlPullParser.END_TAG) {
					if( xpp.getName().equals(CConstants.TAG_LIBRARY_GEOMETRIES) ){
						break; 
					} else{
						parseEndTag(xpp);
					}
				} 
				else if( eventType == XmlPullParser.START_TAG ){
					parseStartTag(xpp);
				}
				else if( eventType == XmlPullParser.TEXT ){
					parseText(xpp);
				}
			}
		
		} catch( XmlPullParserException e ){
			
		} catch( IOException e ){
			
		}
		
		return this; 
	}
	
	protected void parseStartTag(XmlPullParser xpp) throws XmlPullParserException, IOException{
		String tag = xpp.getName(); 
		
		if( tag.equals(CConstants.TAG_GEOMETRY)){
			currentGeometry = new CGeometry();
			
			for( int i=0; i<xpp.getAttributeCount(); i++ ){
				String attr = xpp.getAttributeName(i);
				String val = xpp.getAttributeValue(i);
				
				if( attr.equals(CConstants.ATTR_ID)){
					currentGeometry.id = val.trim(); 
				}
				else if( attr.equals(CConstants.ATTR_NAME)){
					currentGeometry.name = val.trim(); 
				}
			}			
		} else if( tag.equals(CConstants.TAG_SOURCE)){
			currentSource = new CSource(); 
			
			for( int i=0; i<xpp.getAttributeCount(); i++ ){
				String attr = xpp.getAttributeName(i);
				String val = xpp.getAttributeValue(i);
				
				if( attr.equals(CConstants.ATTR_ID)){
					currentSource.id = val; 
				}				
			}
		} else if( tag.equals(CConstants.TAG_SOURCE_FLOAT_ARRAY )){
			if( currentSource != null ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_ID)){
						 // ignore 
					} else if( attr.equals(CConstants.ATTR_COUNT)){
						 currentSource.length = Integer.parseInt(val);
						 currentSource.dataAsFloat = new float[currentSource.length];
						 currentSource.dataType = CDataTypeEnum.Float;
					}
				}
				
				xpp.next(); 
				
				parseDelimitedFloatArrayForSource(currentSource, xpp.getText());
			}						
		} else if( tag.equals(CConstants.TAG_SOURCE_ACCESSOR)){
			if( currentSource != null ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_COUNT)){
						currentSource.count = Integer.parseInt(val); 
					} else if( attr.equals(CConstants.ATTR_STRIDE)){
						currentSource.stride = Integer.parseInt(val);
						currentSource.params = new CSourceParam[currentSource.stride];
					}				
				}
			}					
		} else if( tag.equals(CConstants.TAG_SOURCE_ACCESSOR_PARAM)){
			if( currentSource != null ){
				
				CSourceParam param = new CSourceParam(); 
				
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_NAME)){
						param.name = val;  
					} else if( attr.equals(CConstants.ATTR_TYPE)){
						if( val.equals(CConstants.VALUE_PARAM_FLOAT) ){
							param.type = CDataTypeEnum.Float; 
						} 
					}				
				}
				
				currentSource.params[currentSource.nextParamIndex] = param;
				currentSource.nextParamIndex++;
				
				if( currentSource.nextParamIndex >= currentSource.params.length ){
					CSourceParam[] newParamsArray = new CSourceParam[currentSource.params.length+1];
					System.arraycopy(currentSource.params, 0, newParamsArray, 0, currentSource.params.length);
					currentSource.params = newParamsArray; 
				}
			}					
		} else if( tag.equals(CConstants.TAG_POLYLIST)){
			if( currentGeometry != null ){
				
				currentPolyList = new CPolyList(); 								
				
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_COUNT)){
						currentPolyList.triangles = Integer.parseInt(val);
						//currentPolyList.indexes = new short[currentPolyList.length];
					}				
				}								
			}					
		} else if( tag.equals(CConstants.TAG_INPUT)){
			if( currentPolyList != null ){
								
				CSourceSemantic semantic = null; 
				CSource source = null; 
				int offset = 0; 
				
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_SEMANTIC)){
						if( val.equals(CConstants.VALUE_SEMANTIC_VERTEX)){
							semantic = CSourceSemantic.Vertex; 
						} else if( val.equals(CConstants.VALUE_SEMANTIC_NORMAL)){
							semantic = CSourceSemantic.Normal; 
						} else if( val.equals(CConstants.VALUE_SEMANTIC_TEXCOORD)){
							semantic = CSourceSemantic.TexCoord; 
						}
					} else if( attr.equals(CConstants.ATTR_SOURCE)){
						// remove #
						val = val.replaceAll("#", "");
						// find associated source 
						for( CSource src : currentGeometry.sources ){
							if( src.id.equals(val) ){
								source = src; 
								break;
							}
						}
					} else if( attr.equals(CConstants.ATTR_OFFSET)){
						offset = Integer.parseInt(val);
					}				
				}
				
				if( source != null ){
					CInput input = new CInput();
					input.offset = offset; 
					input.source = source; 
					source.type = semantic; 
					currentPolyList.inputs.add(input);
				}								
			}					
		} else if( tag.equals(CConstants.TAG_P)){
			if( currentPolyList != null ){
				xpp.next(); 
				
				parseDelimitedIndiciesArrayForSource(currentPolyList, xpp.getText());
			}
		} else if( tag.equals(CConstants.TAG_VERTICES)){
			String newId = "";
			
			for( int i=0; i<xpp.getAttributeCount(); i++ ){
				String attr = xpp.getAttributeName(i);
				String val = xpp.getAttributeValue(i);
				
				if( attr.equals(CConstants.ATTR_ID)){
					newId = val; 					
				}
			}
						
			do{
				xpp.next(); // move to the input element
			}while (xpp.getEventType() != XmlPullParser.START_TAG);
						
			if (xpp.getName().equals(CConstants.TAG_INPUT)) {
				for (int i = 0; i < xpp.getAttributeCount(); i++) {
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);

					if (attr.equals(CConstants.ATTR_SOURCE)) {
						val = val.replace("#", "");
						for (CSource source : currentGeometry.sources) {
							if (source.id.equals(val)) {
								source.id = newId;
								break;
							}
						}
					}
				}
			}
		}
	}	
	
	protected void parseText(XmlPullParser xpp){
		
	}
	
	protected void parseEndTag(XmlPullParser xpp){
		String tag = xpp.getName(); 
		
		if( tag.equals(CConstants.TAG_POLYLIST )){
			if( currentPolyList != null ){
				if( currentGeometry != null ){
					currentGeometry.polyList = currentPolyList; 
				}
				
				currentPolyList = null; 
			}
		} else if( tag.equals(CConstants.TAG_SOURCE )){
			if( currentSource != null ){
				if( currentGeometry != null ){
					currentGeometry.sources.add(currentSource);
				}
				
				currentSource = null; 
			}
		} else if( tag.equals(CConstants.TAG_GEOMETRY )){
			if( currentGeometry != null ){				
				geometryLibrary.add(currentGeometry);
				currentGeometry = null; 
			}
		}			
	}	
	
	protected void parseDelimitedIndiciesArrayForSource(CPolyList polyList, String data){
		StringTokenizer parts = new StringTokenizer(data, " ");
		int numTokens = parts.countTokens();
		
		if (numTokens == 0)
			return;
		
		if( polyList.length == 0 ){
			polyList.length = numTokens; 
			polyList.indexes = new short[polyList.length];
		}
		
		for( int i=0; i<numTokens; i++ ){
			String val = parts.nextToken();
			
			polyList.indexes[i] = Short.parseShort(val.trim());
		}
	}
}
