/**
 * CVisualSceneParser.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 23, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.entities.loaders.collada;

import java.io.IOException;
import java.util.List;
import java.util.StringTokenizer;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import com.wmp.nofrills3d.geometry.Bone;
import com.wmp.nofrills3d.utils.Logger;

public class CVisualSceneParser extends CBaseParser {

	public static final String TAG = "CVisualSceneParser";
	
	protected XmlPullParser xpp = null; 
	protected List<Bone> boneLibrary;
	protected Bone currentBone = null; 
	
	protected String currentURL = "";
	
	public static CVisualSceneParser parse(XmlPullParser xpp, List<Bone> boneLibrary){
		return new CVisualSceneParser(xpp, boneLibrary).start(); 
	}
	
	private CVisualSceneParser(XmlPullParser xpp, List<Bone> boneLibrary){
		this.xpp = xpp; 
		this.boneLibrary = boneLibrary; 
	}
	
	public CVisualSceneParser start(){
		if( !xpp.getName().equals(CConstants.TAG_LIBRARY_VISUAL_SCENES) ){
			return this; 
		}
		
		try{
			while(true){
				xpp.next(); 
				int eventType = xpp.getEventType();
				
				if (eventType == XmlPullParser.END_TAG) {
					if( xpp.getName().equals(CConstants.TAG_LIBRARY_VISUAL_SCENES) ){
						break; 
					} else{
						parseEndTag(xpp);
					}
				} 
				else if( eventType == XmlPullParser.START_TAG ){
					parseStartTag(xpp);
				}
				else if( eventType == XmlPullParser.TEXT ){
					parseText(xpp);
				}
			}
		
		} catch( XmlPullParserException e ){
			
		} catch( IOException e ){
			
		}
		
		return this; 
	}
	
	protected void parseStartTag(XmlPullParser xpp) throws XmlPullParserException, IOException{
		String tag = xpp.getName(); 
		
		if( tag.equals(CConstants.TAG_NODE)){
			String id = ""; 
			//String name = ""; 
			String sid = ""; 
			String type = "";
			
			for( int i=0; i<xpp.getAttributeCount(); i++ ){
				String attr = xpp.getAttributeName(i);
				String val = xpp.getAttributeValue(i);
				
				if( attr.equals(CConstants.ATTR_ID)){
					id = val; 
				} else if( attr.equals(CConstants.ATTR_NAME)){
					//name = val; 
				} else if( attr.equals(CConstants.ATTR_SID)){
					sid = val; 
				} else if( attr.equals(CConstants.ATTR_TYPE)){
					type = val; 
				}				
			}
			
			if( type.equals(CConstants.VALUE_NODE_TYPE_JOINT) ){
				Bone bone = new Bone(); 
				bone.id = id; 
				bone.sid = sid; 
				
				if( currentBone != null ){
					currentBone.addChild(bone);
				} 
				
				currentBone = bone; 
			}
		} else if( tag.equals(CConstants.TAG_MATRIX)){
			if( currentBone != null ){
				xpp.next(); 				
				parseDelimitedMatrix(currentBone.localBonetransform, xpp.getText());
			}			
		} else if( tag.equals(CConstants.TAG_INSTANCE_CONTROLLER) ){
				for( int i=0; i<xpp.getAttributeCount(); i++ ){
					String attr = xpp.getAttributeName(i);
					String val = xpp.getAttributeValue(i);
					
					if( attr.equals(CConstants.ATTR_URL)){
						currentURL = val.replace("#", ""); 
					}				
				}			
		} else if( tag.equals(CConstants.TAG_SKELETON) ){
			if( currentURL != null && currentURL.length() > 0 ){
				xpp.next(); 
				String rootBoneId = xpp.getText().replace("#", "");
				
				for(Bone rootBone : boneLibrary ){
					if( rootBone.id.equals(rootBoneId) ){
						rootBone.url = currentURL; 
					}
				}
			}
		}
		
	}
	
	protected void parseText(XmlPullParser xpp){
		
	}
	
	protected void parseEndTag(XmlPullParser xpp){
		String tag = xpp.getName();
		
		if( tag.equals(CConstants.TAG_NODE)){	
			if( currentBone != null ){
			
				if( currentBone.parent != null ){ // pop current bone of the stack if we don't have a root
					currentBone = currentBone.parent;
				} else{ // else add it to our collection
					boneLibrary.add(currentBone);
					currentBone = null; 
				}
			}
		} else if(tag.equals(CConstants.TAG_INSTANCE_CONTROLLER) ){
			currentURL = "";
		}
	}	
}
