package com.wmp.nofrills3d.entities.loaders.md5;

import java.util.StringTokenizer;

import com.wmp.nofrills3d.math.Quaternion;
import com.wmp.nofrills3d.math.Vector2;
import com.wmp.nofrills3d.math.Vector3;
import com.wmp.nofrills3d.utils.Logger;

public abstract class MD5BaseParser {
	
	public static final String TAG = "MD5BaseParser";
	
	protected boolean flipYZ = false; 
	
	/**
	 * expecting: 0.0111
	 * @param parts
	 * @return
	 */
	protected float readFloat(StringTokenizer parts){
		return Float.parseFloat(parts.nextToken());
	}
	
	/**
	 * expecting: -1
	 * @param parts
	 * @return
	 */
	protected int readInt(StringTokenizer parts){
		return Integer.parseInt(parts.nextToken());
	}
	
	/**
	 * expecting: "Bone"
	 * @param parts
	 * @return
	 */
	protected String readString(StringTokenizer parts){
		String part = parts.nextToken(); 
		
		return part.replaceAll("\"", "").trim();  
	}

	/**
	 * expecting: ( 0.000000 0.000000 0.000000 ) 
	 * @param parts
	 * @return
	 */
	protected Vector3 readVector3(StringTokenizer parts){		
		 
		String openParam = parts.nextToken(); 
		float x = Float.parseFloat(parts.nextToken());
		float y = Float.parseFloat(parts.nextToken());
		float z = Float.parseFloat(parts.nextToken());
		String closeParam = parts.nextToken(); 
		
		if( !openParam.contains("(") || !closeParam.contains(")") ){
			// TODO raise a validation error
			if( Logger.Logging ){
				Logger.LogWarning(TAG, "failed to parse Vector3");
			}			
			return null; 
		}		
		
		return new Vector3(x, (flipYZ ? z : y), (flipYZ ? y : z));
	}
	
	/**
	 * expecting: ( 0.000000 0.000000 0.000000 ) 
	 * @param parts
	 * @return
	 */
	protected float[] readVector3AsFloatArray(StringTokenizer parts){		
		 
		String openParam = parts.nextToken(); 
		float x = Float.parseFloat(parts.nextToken());
		float y = Float.parseFloat(parts.nextToken());
		float z = Float.parseFloat(parts.nextToken());
		String closeParam = parts.nextToken(); 
		
		if( !openParam.contains("(") || !closeParam.contains(")") ){
			// TODO raise a validation error
			if( Logger.Logging ){
				Logger.LogWarning(TAG, "failed to parse Vector3");
			}			
			return null; 
		}		
		
		return new float[]{x,(flipYZ ? z : y), (flipYZ ? y : z)}; 
	}
	
	/**
	 * expecting: ( 0.000000 0.000000 ) 
	 * @param parts
	 * @return
	 */
	protected Vector2 readVector2(StringTokenizer parts){		
		 
		String openParam = parts.nextToken(); 
		float x = Float.parseFloat(parts.nextToken());
		float y = Float.parseFloat(parts.nextToken());
		String closeParam = parts.nextToken(); 
		
		if( !openParam.contains("(") || !closeParam.contains(")") ){
			// TODO raise a validation error
			if( Logger.Logging ){
				Logger.LogWarning(TAG, "failed to parse Vector3");
			}			
			return null; 
		}		
		
		return new Vector2(x, y);
	}
	
	/**
	 * expecting: ( 0.000000 0.000000 ) 
	 * @param parts
	 * @return
	 */
	protected float[] readVector2AsFloatArray(StringTokenizer parts){		
		 
		String openParam = parts.nextToken(); 
		float x = Float.parseFloat(parts.nextToken());
		float y = Float.parseFloat(parts.nextToken());
		String closeParam = parts.nextToken(); 
		
		if( !openParam.contains("(") || !closeParam.contains(")") ){
			// TODO raise a validation error
			if( Logger.Logging ){
				Logger.LogWarning(TAG, "failed to parse Vector3");
			}			
			return null; 
		}		
		
		return new float[]{x,y}; 
	}
	
	/**
	 * expecting: ( -0.707107 -0.000000 -0.000000 )	
	 * @param parts
	 * @return
	 */
	protected Quaternion readQuaternion(StringTokenizer parts){
		
		String openParam = parts.nextToken(); 
		float x = Float.parseFloat(parts.nextToken());
		float y = Float.parseFloat(parts.nextToken());
		float z = Float.parseFloat(parts.nextToken());
		String closeParam = parts.nextToken(); 
		
		if( !openParam.contains("(") || !closeParam.contains(")") ){
			// TODO raise a validation error
			if( Logger.Logging ){
				Logger.LogWarning(TAG, "failed to parse Quaternion");
			}			
			return null; 
		}		
		
		return new Quaternion(x, y, z);
	}
	
	/**
	 * expecting: ( -0.707107 -0.000000 -0.000000 )	
	 * @param parts
	 * @return
	 */
	protected float[] readQuaternionAsFloatArray(StringTokenizer parts){
		
		String openParam = parts.nextToken(); 
		float x = Float.parseFloat(parts.nextToken());
		float y = Float.parseFloat(parts.nextToken());
		float z = Float.parseFloat(parts.nextToken());
		String closeParam = parts.nextToken(); 
		
		if( !openParam.contains("(") || !closeParam.contains(")") ){
			// TODO raise a validation error
			if( Logger.Logging ){
				Logger.LogWarning(TAG, "failed to parse Quaternion");
			}			
			return null; 
		}		
		
		return new float[]{x, y, z, Quaternion.computeW(x, y, z)}; 
	}
	
}
