package com.wmp.nofrills3d.entities.loaders.md5;

import com.wmp.nofrills3d.math.MathHelper;
import com.wmp.nofrills3d.math.Quaternion;
import com.wmp.nofrills3d.math.Vector3;

public class MD5Mesh {

	public int vertCount = 0; 
	public int triCount = 0; 
	public int weightCount = 0; 
	
	public MD5Vert[] verts = null; 
	public int[] tris = null;
	public MD5JointWeight[] weights = null; 
	
	public MD5Mesh(){
		
	}
	
	public void calculatePositions(MD5Joint[] joints){
		float[] position = new float[]{0.0f, 0.0f, 0.0f}; 
		//float[] transformedPosition = new float[]{0.0f, 0.0f, 0.0f};
		
		Quaternion tmpQuaternion = new Quaternion(); 
		Vector3 tmpVector3 = new Vector3();
		Vector3 tmpTransformedVector3 = new Vector3();
		
		for( int i=0; i<verts.length; i++ ){
			position[0] = 0.0f; position[1] = 0.0f; position[2] = 0.0f;
			
			MD5Vert vert = verts[i]; 						
			
			for( int j=0; j<vert.weightCount; j++ ){
				MD5JointWeight weight = weights[vert.weightIndex + j];
				MD5Joint joint = joints[weight.jointIndex];
				
				///
				
//				// calculate transfomed vertex for this weight 				
//				MathHelper.rotateAroundPoint(weight.position, joint.orientation, transformedPosition);
//				
//				// sum all weights (should be equal to 1.0) 
//				position[0] += (joint.position[0] + transformedPosition[0]) * weight.weight;
//				position[1] += (joint.position[1] + transformedPosition[1]) * weight.weight;
//				position[2] += (joint.position[2] + transformedPosition[2]) * weight.weight;
				
				///
				
				tmpVector3.set(weight.position);
				tmpQuaternion.set(joint.orientation);
				
				Quaternion.multiply(tmpQuaternion, tmpVector3, tmpTransformedVector3);												
				
				position[0] += (joint.position[0] + tmpTransformedVector3.x) * weight.weight;
				position[1] += (joint.position[1] + tmpTransformedVector3.y) * weight.weight;
				position[2] += (joint.position[2] + tmpTransformedVector3.z) * weight.weight;
			}
			
			verts[i].position = new float[]{position[0], position[1], position[2]}; 
		}
	}	
	
	/**
	 * calulates normals for each vert - the positions must be first set (calcualtePositions) 
	 */
	public void calcualteNormals(){		
		
		Vector3 v0 = new Vector3(); 
		Vector3 v1 = new Vector3();
		Vector3 v2 = new Vector3();
		Vector3 normal = new Vector3(); 
		
		/*
		 * Set Vector U to (Triangle.p2 minus Triangle.p1)
        Set Vector V to (Triangle.p3 minus Triangle.p1)
 
        Set Normal.x to (multiply U.y by V.z) minus (multiply U.z by V.y)
        Set Normal.y to (multiply U.z by V.x) minus (multiply U.x by V.z)
        Set Normal.z to (multiply U.x by V.y) minus (multiply U.y by V.x)
		 */
		
		for( int i=0; i<tris.length; i+=3 ){
			v0.set(verts[tris[i+0]].position);
			v1.set(verts[tris[i+1]].position);
			v2.set(verts[tris[i+2]].position);
			
			Vector3.crossProduct(Vector3.subtract(v1, v0), Vector3.subtract(v2, v0), normal);
			
//			Vector3 u  = Vector3.subtract(v1, v0);
//	        Vector3 v = Vector3.subtract(v2, v0);
//	 
//	        normal.x = (u.y * v.z) - (u.z * v.y);
//	        normal.y = (u.z * v.x) - (u.x * v.z);
//	        normal.z = (u.x * v.y) - (u.y * v.x);
//			
			normal.normalise();
			
			verts[tris[i+0]].normal = new float[]{normal.x, normal.y, normal.z};
			verts[tris[i+1]].normal = new float[]{normal.x, normal.y, normal.z}; 
			verts[tris[i+2]].normal = new float[]{normal.x, normal.y, normal.z}; 
		}
	}
}
