package com.wmp.nofrills3d.entities.loaders.md5;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.StringTokenizer;

import com.wmp.nofrills3d.math.Quaternion;
import com.wmp.nofrills3d.math.Vector3;

public class MD5MeshParser extends MD5BaseParser{

	public MD5MeshParser(){
		
	}
	
	public MD5Data parse(InputStream is) throws IOException{
		MD5Data md5Data = new MD5Data(); 
		
		BufferedReader in = null; 
		String line = "";
		
		int meshIndex = 0; 
		
		in = new BufferedReader(new InputStreamReader(is));
		
		while ((line = in.readLine()) != null) {
			//line = line.replace("\t", "");
			
			StringTokenizer parts = new StringTokenizer(line, " ");
			int numTokens = parts.countTokens();
			
			if (numTokens == 0)
				continue;
			
			String tag = parts.nextToken();
			
			if( tag.equals(MD5Constants.VERSION)){
				md5Data.version = parts.nextToken().trim(); 
			}
			else if( tag.equals(MD5Constants.NUM_JOINTS)){
				md5Data.numJoints = readInt(parts);
				md5Data.joints = new MD5Joint[md5Data.numJoints];
			}
			else if( tag.equals(MD5Constants.NUM_MESHES)){
				md5Data.numMeshes = readInt(parts);
				md5Data.meshes = new MD5Mesh[md5Data.numMeshes];
			}
			else if( tag.equals(MD5Constants.JOINTS)){
				parseJoints(in, md5Data);
			}
			else if( tag.equals(MD5Constants.MESH) ){
				parseMesh(in, md5Data, meshIndex);
				meshIndex++; 
			}
			
		}
		
		in.close(); 				
		
		return md5Data; 
	}
	
	/**
	 * 
	 * @param in
	 * @throws IOException 
	 */
	protected void parseMesh(BufferedReader in, MD5Data md5Data, int meshIdx) throws IOException{
		MD5Mesh mesh = new MD5Mesh(); 		
		
		String line = "";
		
		while ((line = in.readLine()) != null) {
			line = line.replace("\t", " ");
			
			StringTokenizer parts = new StringTokenizer(line, " ");
			int numTokens = parts.countTokens();
			
			if (numTokens == 0)
				continue;
			
			String tag = parts.nextToken();
			
			if( tag.equals("}") ){
				break; 
			}
			else if( tag.equals(MD5Constants.NUM_VERTS) ){
				int numVerts = readInt(parts);
				mesh.vertCount = numVerts; 
				mesh.verts = new MD5Vert[numVerts];
			} 
			else if( tag.equals(MD5Constants.VERT) ){
				int vertIndex = readInt(parts);				
				float[] uv = readVector2AsFloatArray(parts);
				// flip y 
				uv[1] = 1.0f - uv[1]; 
				int weightIndex = readInt(parts);
				int weightCount = readInt(parts);
				MD5Vert vert = new MD5Vert(uv, weightIndex, weightCount);
				mesh.verts[vertIndex] = vert; 
			}
			else if( tag.equals(MD5Constants.NUM_TRIS) ){
				int triCount = readInt(parts);
				mesh.triCount = triCount; 
				mesh.tris = new int[triCount * 3];				
			}
			else if( tag.equals(MD5Constants.TRI) ){
				int triIndex = readInt(parts);
				int startIdx = triIndex * 3;
				// counter clock-wise 
				mesh.tris[startIdx + 0] = readInt(parts);
				mesh.tris[startIdx + 2] = readInt(parts);
				mesh.tris[startIdx + 1] = readInt(parts);				
			}
			else if( tag.equals(MD5Constants.NUM_WEIGHTS) ){
				int weightCount = readInt(parts);
				mesh.weightCount = weightCount; 
				mesh.weights = new MD5JointWeight[weightCount];
			}
			else if( tag.equals(MD5Constants.WEIGHT) ){
				// weight 0 1 1.000000 ( -0.147061 0.297007 0.147061 ) (index, joint index, weight, position 
				int weightIndex = readInt(parts);
				int jointIndex = readInt(parts);
				float weight = readFloat(parts);
				float[] position = readVector3AsFloatArray(parts);
				mesh.weights[weightIndex] = new MD5JointWeight(jointIndex, weight, position);
			}
			
		}
		
		md5Data.meshes[meshIdx] = mesh;
	}
	
	/**
	 * expecting: 
	 * 	"Bone"	-1 ( 0.000000 0.000000 0.000000 ) ( -0.707107 -0.000000 -0.000000 )
	 * 	...
	 * }	
	 * @param parts
	 * @param md5Data
	 * @throws IOException 
	 */
	private void parseJoints(BufferedReader in, MD5Data md5Data) throws IOException{
		String line = "";
		
		int jointIdx = 0; 
		
		while ((line = in.readLine()) != null) {
			line = line.replace("\t", " ");
			
			StringTokenizer parts = new StringTokenizer(line, " ");
			int numTokens = parts.countTokens();
			
			if (numTokens == 0)
				continue;
			
			String tag = parts.nextToken();
			
			if( tag.equals("}") ){
				break; 
			} else{
				String name = tag.replaceAll("\"", "").trim();
				int parentIdx = readInt(parts);
				float[] verts = readVector3AsFloatArray(parts);
				float[] orientation = readQuaternionAsFloatArray(parts);
				
				md5Data.joints[jointIdx] = new MD5Joint(name, parentIdx, verts, orientation);
				jointIdx++; 
			}
		}
					
	}
	
}
