/**
 * Bone.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 23, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.geometry;

import java.util.List;
import java.util.Vector;

import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.math.Vector3;

public class Bone {

	public String url = ""; // used by the parent bone to associate it with the skin
	
	public Bone parent = null; 
	
	public List<Bone> children = new Vector<Bone>();
	
	public int index = -1; 
	
	public String id = ""; 
	
	public String sid = "";
	
	public Matrix4 localBonetransform = Matrix4.createIdentity();
	
	public Matrix4 invBoneTransform = Matrix4.createIdentity();
	
	protected boolean locked = false; 
	
	protected long lockChangeTimestamp = 0;
	
	public long lockBlendTime = 500; 
		
	public Bone(){
		
	}		
	
	public void addChild(Bone bone){
		bone.parent = this; 
		children.add(bone);
	}
	
	public Bone findBoneAtIndex(int index){
		if( this.index == index ){
			return this; 
		} else{
			for( Bone child : children ){
				Bone bone = child.findBoneAtIndex(index);
				if( bone != null ){
					return bone; 
				}
			}
		}
		
		return null; 
	}
	
	public Bone findBoneWithId(String id){
		if( this.id.equalsIgnoreCase(id) ){
			return this; 
		} else{
			for( Bone child : children ){
				Bone bone = child.findBoneWithId(id);
				if( bone != null ){
					return bone; 
				}
			}
		}
		
		return null; 
	}
	
	public Bone getParent(int step){
		if( parent == null || step == 0 ){
			return this; 
		} else{
			return parent.getParent(--step);
		}		
	}
	
	public long getLockChangedTimestamp(){
		return lockChangeTimestamp;
	}
	
	public boolean isLocked(){
		return locked; 
	}
	
	public void setLock(boolean locked, int step){
		step--; 
		this.locked = locked;
		
		lockChangeTimestamp = System.currentTimeMillis(); // used for animation blending  
		
		if( step >= 0 && parent != null ){
			parent.setLock(locked, step);
		}
	}
	
	public int size(){
		int size = 1; 
		
		for(Bone child : children ){
			size += child.size(); 
		}
		
		return size; 
	}				
	
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();		
		sb.append(id);
		sb.append(" ("); sb.append(index); sb.append(")");
		
		if( children.size() > 0 ){
			sb.append(" [");
			for( Bone child : children ){
				sb.append(" ");
				sb.append(child.toString());
			}
			sb.append("]");
		}
		
		return sb.toString(); 
	}
}
