package com.wmp.nofrills3d.geometry;

import com.wmp.nofrills3d.scene.Constants;
import com.wmp.nofrills3d.utils.Logger;


public final class PrimitiveMeshFactory {

	public static final float PI = (float)Math.PI;  
	
	public static Mesh createSphere(float radius, int segmentsW, int segmentsH ){
		int numVertices = (segmentsW + 1) * (segmentsH + 1);
		int numIndices = 2 * segmentsW * (segmentsH - 1) * 3;

		float[] vertices = new float[numVertices * 3];
		float[] normals = new float[numVertices * 3];
		float[] colours = new float[numVertices * 4];
		short[] indices = new short[numIndices];

		int i, j;
		int vertIndex = 0, index = 0;
		final float normLen = 1.0f / radius;

		for (j = 0; j <= segmentsH; ++j) {
			float horAngle = PI * j / segmentsH;
			float z = radius * (float) Math.cos(horAngle);
			float ringRadius = radius * (float) Math.sin(horAngle);

			for (i = 0; i <= segmentsW; ++i) {
				float verAngle = 2.0f * PI * i / segmentsW;
				float x = ringRadius * (float) Math.cos(verAngle);
				float y = ringRadius * (float) Math.sin(verAngle);

				normals[vertIndex] = x * normLen;
				vertices[vertIndex++] = x;
				normals[vertIndex] = z * normLen;
				vertices[vertIndex++] = z;
				normals[vertIndex] = y * normLen;
				vertices[vertIndex++] = y;

				if (i > 0 && j > 0) {
					short a = (short)((segmentsW + 1) * j + i);
					short b = (short)((segmentsW + 1) * j + i - 1);
					short c = (short)((segmentsW + 1) * (j - 1) + i - 1);
					short d = (short)((segmentsW + 1) * (j - 1) + i);

					if (j == segmentsH) {
						indices[index++] = a;
						indices[index++] = c;
						indices[index++] = d;
					} else if (j == 1) {
						indices[index++] = a;
						indices[index++] = b;
						indices[index++] = c;
					} else {
						indices[index++] = a;
						indices[index++] = b;
						indices[index++] = c;
						indices[index++] = a;
						indices[index++] = c;
						indices[index++] = d;
					}
				}
			}
		}

		int numUvs = (segmentsH + 1) * (segmentsW + 1) * 2;
		float[] textureCoords = new float[numUvs];

		numUvs = 0;
		for (j = 0; j <= segmentsH; ++j) {
			for (i = 0; i <= segmentsW; ++i) {
				textureCoords[numUvs++] = -(float) i / segmentsW;
				textureCoords[numUvs++] = (float) j / segmentsH;
			}
		}

//		int numColors = numVertices * 4;
//		for (j = 0; j < numColors; j += 4)
//		{
//			colours[j] = 1.0f;
//			colours[j + 1] = 0;
//			colours[j + 2] = 0;
//			colours[j + 3] = 1.0f;
//		}
		
		// build mesh 
		Mesh mesh = new Mesh("Sphere");
		
		VertexDefinition vertexDef = new VertexDefinition(Constants.ATTRIBUTE_POSITION, 3);
		VertexDefinition normalDef = new VertexDefinition(Constants.ATTRIBUTE_NORMAL, 3);
		VertexDefinition texCoordsDef = new VertexDefinition(Constants.ATTRIBUTE_TEX_COORD, 2);
//		VertexDefinition colourDef = new VertexDefinition(Constants.ATTRIBUTE_COLOUR, 4);		
		
//		mesh.pushVertexDefinition(vertexDef, normalDef, texCoordsDef, colourDef);
		mesh.pushVertexDefinition(vertexDef, normalDef, texCoordsDef);
				
		//float[] vertexArray = new float[(numVertices * 3) + (numVertices * 3) + numUvs + (numVertices * 4)];
		float[] vertexArray = new float[(numVertices * 3) + (numVertices * 3) + numUvs];
		
		int vertexIndex = 0; 
		
		for( int idx=0; idx<numVertices; idx++ ){ 
			
			// vertices positions
			vertexArray[vertexIndex++] = (float)vertices[idx * 3 + 0]; // position x
			vertexArray[vertexIndex++] = (float)vertices[idx * 3 + 1]; // position y
			vertexArray[vertexIndex++] = (float)vertices[idx * 3 + 2]; // position z
			
			// normals 
			vertexArray[vertexIndex++] = (float)normals[idx * 3 + 0]; // x
			vertexArray[vertexIndex++] = (float)normals[idx * 3 + 1]; // y
			vertexArray[vertexIndex++] = (float)normals[idx * 3 + 2]; // z
			
			// uv's 
			vertexArray[vertexIndex++] = (float)textureCoords[idx * 2 + 0]; // u
			vertexArray[vertexIndex++] = (float)textureCoords[idx * 2 + 1]; // v
			
			// colours
//			vertexArray[vertexIndex++] = (float)colours[idx * 4 + 0]; // r
//			vertexArray[vertexIndex++] = (float)colours[idx * 4 + 1]; // g
//			vertexArray[vertexIndex++] = (float)colours[idx * 4 + 2]; // b
//			vertexArray[vertexIndex++] = (float)colours[idx * 4 + 3]; // a
		}
		
		mesh.setIndiciesData(indices);
		mesh.setVertexData(vertexArray);
		
		return mesh;
	}
}
