/**
 * SkinnedMesh.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 15, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.geometry;

import java.util.List;

import com.wmp.nofrills3d.animations.AnimationClip;
import com.wmp.nofrills3d.entities.loaders.collada.CVertexBoneInfluence;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.math.Vector3;
import com.wmp.nofrills3d.physics.BoxCollider;
import com.wmp.nofrills3d.scene.Constants;

public class SkinnedMesh extends Mesh {

	public Bone rootBone = null;
	
	public int[] boneHierarchy = null;
	
	public BoxCollider[] boneBoundingBox;
	
	public Matrix4[] bindPose = null; 
	
	public Matrix4[] inverseBindPose;		
	
	public List<AnimationClip> animationClips = null; 
	
	/**
	 * @param tag
	 */
	public SkinnedMesh(String tag) {
		super(tag);
		
	}
	
	protected void buildBoundingBoxFromVertices(float[] data){
		
		VertexDefinition vd = findVertexDefinitionWithName(Constants.ATTRIBUTE_POSITION);
		
		if( vd == null ){
			return; 
		}
		
		int count = vd.count; 
		
		Vector3 min = new Vector3(Float.MIN_VALUE);
		Vector3 max = new Vector3(Float.MAX_VALUE);
		
		Vector3 vPos = Vector3.createZero(); 
		
		for( int i=0; i<vertexLength; i+=stride){
						
			vPos.x = data[i]; 
			if( count >= 2 ) vPos.y = data[i+1];
			if( count == 3 ) vPos.z = data[i+2];
			
			min = Vector3.min(min, vPos);
			max = Vector3.max(max, vPos);
			
		}

		boundingBox = new BoxCollider(null, min, max);
		
	}
	
	public void copyBindPose(Matrix4[] transforms){
		if( transforms == null || transforms.length != bindPose.length ){
			return; 
		}
		
		int boneCount = bindPose.length; 
		
		for( int i=0; i<boneCount; i++ ){
			transforms[i].set(bindPose[i].matrix);					
		}
	}
	
	public String bindPoseToString(){
		StringBuffer sb = new StringBuffer();
		
		int boneCount = bindPose.length; 
		
		for( int i=0; i<boneCount; i++ ){
			sb.append(String.format("%d -> %s", i, bindPose[i].toString())); 
		}
		
		return sb.toString(); 
	}		
}
