package com.wmp.nofrills3d.ik;

import com.wmp.nofrills3d.entities.SkinnedModel;
import com.wmp.nofrills3d.geometry.Bone;
import com.wmp.nofrills3d.geometry.SkinnedMesh;
import com.wmp.nofrills3d.math.MathHelper;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.math.Vector3;

public abstract class IKConstraint {

	public int id = -1; 
	
	public String tag = "IKConstraint";
	
	public IKConstraintDelegate delegate = null;
	
	public float ikSpeed = 0.2f;
	
	protected SkinnedModel skinnedModel;
	
	protected SkinnedMesh skinnedMesh; 
	
	protected Bone ikBone;
	
	protected Vector3 target = Vector3.createZero();
	
	protected Vector3 targetWorld = Vector3.createZero();
	
	protected boolean enabled = false;
	
	protected int chainLength;		
	
	public IKConstraint(SkinnedModel model, Bone bone, int length){
		this.skinnedModel = model;
		this.ikBone = bone; 
		this.chainLength = length;	
		
		this.skinnedMesh = (SkinnedMesh)this.skinnedModel.mesh; 
	}	
	
	public boolean isActive(){
		return enabled; 
	}
	
	public void activate(){
		if( enabled ){
			return; 
		}
		
		enabled = true; 
		
		ikBone.setLock(true, chainLength);
	}
	
	public void deactivate(){
		deactivate(true);				
	}
	
	public void deactivate(boolean releaseLock){
		if( !enabled ){
			return; 
		}
		
		enabled = false; 
		
		if( releaseLock ){
			ikBone.setLock(false, chainLength);
		}
	}
	
	public void update(float elapsedTime){
		if( !enabled ){
			return; 
		}
		
		// update target 
		target.set(targetWorld);
		skinnedModel.transformFromWorldToModelSpace(target);
		
		if( solveIK(elapsedTime) ){
			onTargetReached(); 
		}
	}
	
	protected void onTargetReached(){
		if( delegate != null ){
			delegate.onTargetReached(this);
		}
	}
	
	protected abstract boolean solveIK(float elapsedTime);		
	
//	protected float calculateAngle(Vector3 v0, Vector3 v1, Vector3 crossProductOfV0andV1){
//		Vector3 n0 = new Vector3(v0).normalise(); 
//		Vector3 n1 = new Vector3(v1).normalise();
//		Vector3 nCross = new Vector3(); 
//		Vector3.crossProduct(n0, n1, nCross);
//		
//		nCross.normalise(); 
//                
//        float nDot = Vector3.dotProduct(n0, n1);
//        
//        if (Float.isNaN(nDot)){
//        	nDot = 0;
//        }
//        
//        if (nDot > 1) {
//        	nDot = 1;
//        }
//        
//        if (nDot < -1){
//        	nDot = -1;
//        }
//        
//        float a = (float)Math.acos(nDot);
//        
//        if (Vector3.add(n0, n1).length() < 0.01f){
//        	return (float)Math.PI;
//        }
//        
//        return Vector3.dotProduct(nCross, crossProductOfV0andV1) >= 0 ? a : -a;
//    }
	
	public void setTarget(Vector3 target){
		setTarget(target, true);
	}
	
	public void setTarget(Vector3 target, boolean worldSpace){		
		if( worldSpace ){
			this.targetWorld.set(target);
			this.target.set(target);
			skinnedModel.transformFromWorldToModelSpace(this.target);			
		} else{
			this.target.set(target);			
			this.targetWorld.set(target);
			skinnedModel.transformFromModelToWorldSpace(this.targetWorld);			
		}
	}		
	
	public Vector3 getTarget(){
		return getTarget(true);
	}
	
	public Vector3 getTarget(boolean worldSpace){		
		if( worldSpace ){		
			return targetWorld.clone();
		} else{
			return target.clone(); 
		} 
	}
	
	public SkinnedModel getModel(){
		return skinnedModel;
	}
	
	/** 
	 *  effector is the end point for a bone (in model space)
	 **/ 
//	public Vector3 getEffectorForBoneAtIndex(int index){
//		int boneCount = skinnedMesh.boneHierarchy.length; 
//		
//		int childIndex = -1; 
//		
//		for( int i = index+1; i<boneCount; i++ ){
//			if( skinnedMesh.boneHierarchy[i] == index ){
//				childIndex = i; 
//			}
//		}
//		
//		if( childIndex != -1 ){
//			Matrix4 childBoneTransform = skinnedModel.worldTransforms[index];
//			return Matrix4.decomposeTranslation(childBoneTransform);						
//		} else{
//			Matrix4 boneTransform = skinnedModel.worldTransforms[index];
//			return Matrix4.decomposeTranslation(boneTransform);
//		}
//	}
	
//	public Vector3 getDestinationForBoneAtIndex( int index ){							 
//		int boneCount = skinnedMesh.boneHierarchy.length; 
//		
//		int childIndex = -1; 
//		
//		for( int i = index+1; i<boneCount; i++ ){
//			if( skinnedMesh.boneHierarchy[i] == index ){
//				childIndex = i; 
//			}
//		}
//		
//		if( childIndex != -1 ){
//			Matrix4 boneTransform = skinnedModel.worldTransforms[index];
//			Vector3 startPoint = Matrix4.decomposeTranslation(boneTransform);
//			
//			Matrix4 childBoneTransform = skinnedModel.worldTransforms[childIndex];
//			Vector3 endPoint = Matrix4.decomposeTranslation(childBoneTransform);
//			
//			return Vector3.subtract(endPoint, startPoint);
//			
//		} else{
//			return Vector3.createZero();
//		}
//	}
	
}
