package com.wmp.nofrills3d.ik;

import com.wmp.nofrills3d.entities.SkinnedModel;
import com.wmp.nofrills3d.geometry.Bone;
import com.wmp.nofrills3d.math.MathHelper;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.math.Quaternion;
import com.wmp.nofrills3d.math.Vector3;
import com.wmp.nofrills3d.utils.Logger;

public class IKPivot extends IKConstraint {	
	
	private float rotationChangeThreshold = 0.20943951f;
		
	protected Vector3 bindRotation = Vector3.createZero();
	
	public IKPivot(SkinnedModel model, Bone bone) {
		super(model, bone, 1);
	}

	@Override
	protected boolean solveIK(float elapsedTime) {
		
		Vector3 goalInBoneWorldSpace = Vector3.createZero(); 
		Vector3 endEffectorInBoneWorldSpace = Vector3.createZero();
		
		Matrix4 rotationAdjustment = Matrix4.createIdentity(); 
		Matrix4 tmp = Matrix4.createIdentity(); 
		
		Matrix4 curBoneWorld = skinnedModel.worldTransforms[ikBone.index];		
		Matrix4.decomposeTranslation(curBoneWorld, endEffectorInBoneWorldSpace);		
		//Vector3.transform( target, Matrix4.invert(curBoneWorld, curBoneWorldInv), goalInBoneWorldSpace);
		goalInBoneWorldSpace.set(target);
		
		//goalInBoneWorldSpace.x = endEffectorInBoneWorldSpace.x + 5; 
		//goalInBoneWorldSpace.y = endEffectorInBoneWorldSpace.y + 5;
		//goalInBoneWorldSpace.z = endEffectorInBoneWorldSpace.z; 		
	
		Vector3 _forward = Vector3.subtract(endEffectorInBoneWorldSpace, goalInBoneWorldSpace).normalise();
		Vector3 _currentForward = skinnedModel.boneTransforms[ikBone.index].getForward().normalise(); 
		
		float _angle = MathHelper.angleBetween(_forward, skinnedModel.boneTransforms[ikBone.index].getForward().normalise());
		
		if( _angle > rotationChangeThreshold ){
        
			Vector3 _axis = Vector3.crossProduct(_currentForward, _forward);
			_axis.normalise();
			
			_axis.z = MathHelper.clamp(_axis.z, -0.1f, 0.1f);
			_axis.z = 0.0f; 
			_axis.normalise();				
			
			rotationAdjustment.reset(); 
			Matrix4.createFromAxisAngle(_axis, _angle, rotationAdjustment);
				
			Matrix4.multiply(skinnedModel.boneTransforms[ikBone.index], rotationAdjustment, tmp);
	        Matrix4.blend(skinnedModel.boneTransforms[ikBone.index], tmp, ikSpeed, skinnedModel.boneTransforms[ikBone.index]);	        	        
	        
	        return false;
		}
        
		return true;
	}		
}
