package com.wmp.nofrills3d.input;

import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;

import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;

public class UserInputManager implements OnTouchListener {
	
	public boolean enabled = true; 
	
	protected ArrayBlockingQueue<UserInputEvent> inputQueue = new ArrayBlockingQueue<UserInputEvent>(20);	
	
	protected View view;
	
	protected Object inputLock = new Object();
	
	public UserInputManager(View view){
		this.view = view;
		
		this.view.setOnTouchListener(this);
	}
	
	public void onPause(){
		this.view.setOnTouchListener(null);
	}
	
	public void onResume(){
		view.setOnTouchListener(this);
	}
	
	public boolean getInputEvents(List<UserInputEvent> dst){		
		if( dst == null ){
			return false; 
		}
		
		dst.clear(); 
		
		synchronized(inputLock) {
			while (!inputQueue.isEmpty()) {
				UserInputEvent input;
				try {
					input = inputQueue.take(); 
					
					dst.add(input);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}				
			}			
		}
		
		return true; 
	}

	@Override
	public boolean onTouch(View view, MotionEvent event) {
		if( !enabled ){
			return false; 
		}
		
		int hist = event.getHistorySize();
		if (hist > 0) {
			for (int i = 0; i < hist; i++) {						
				UserInputEvent input = new UserInputEvent();
				input.initWithEventHistory(event, i);
				AddInputEvent( input );
			}
		}
			
		UserInputEvent input = new UserInputEvent(); 
		input.initWithEvent(event);
		AddInputEvent( input ); 
		
		try {
			Thread.sleep(16);
		} catch (InterruptedException e) {
		}
		
		return true;
	}
	
	private void AddInputEvent( UserInputEvent input ){
		synchronized(inputLock) {
			inputQueue.add(input);
		}
	}

}
