/**
 * NoFrills3D
 * com.wmp.nofrills3d.material
 * 
 * @author Joshua Newnham
 * @copyright We Make Play Ltd. 2013 
 * 
 */
package com.wmp.nofrills3d.material;

import static android.opengl.GLES20.glUniformMatrix4fv;
import static android.opengl.Matrix.setLookAtM;
import android.content.Context;
import android.opengl.GLES20;
import android.opengl.Matrix;

import com.wmp.colourexplosion.R;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.scene.Constants;

/**
 * 
 */
public class ShadowMaterial extends Material{

	private Shader depthShader = null; 
	
	private Matrix4 lightProjectionMatrix = new Matrix4();
	
	private Matrix4 lightModelViewMatrix = new Matrix4(); 	
	
	private boolean usingDepthShader = false; 
	
	public Texture shadowMapTex = null; 
	
	/**
	 * @param context
	 * @param vertexShaderResId
	 * @param fragmentShaderResId
	 */
	public ShadowMaterial(Context context, int vertexShaderResId, int fragmentShaderResId) {		
		super(context, vertexShaderResId, fragmentShaderResId);	
		
		initDepthShader(context);
		
		//this.depthMapTex = depathMapTex; 
	}
	
	protected void initDepthShader(Context context){
		depthShader = new Shader(context, R.raw.depth_vs, R.raw.depth_fs);
	}
	
	public void setShadowMapTexture(Texture texture){
		this.shadowMapTex = texture;
		if( this.shadowMapTex != null ){
			//lightProjectionMatrix = Matrix4.createPerspective(70.0f, (float)this.shadowMapTex.width/(float)this.shadowMapTex.height, 1.0f, 5000.0f);
			float ratio = (float)this.shadowMapTex.width/(float)this.shadowMapTex.height;
			Matrix.frustumM(lightProjectionMatrix.matrix, 0, -ratio, ratio, -1, 1, 1f, 40000);
		}
	}
	
	@Override
	public int bind(){
		usingDepthShader = false;
		int currentProgram = -1; 
		
		if( this.pass == 0 ){
			usingDepthShader = true; 
			
			currentProgram = depthShader.bind();						
			
			// View from the light's perspective 
			setLookAtM(lightModelViewMatrix.matrix, 0, 
					light.lightPosition[0], light.lightPosition[1], light.lightPosition[2], 
					light.lightPosition[4], light.lightPosition[5], light.lightPosition[6], 
					light.lightPosition[7], light.lightPosition[8], light.lightPosition[9]);
			
			
			Matrix4.multiply(viewTransform, lightModelViewMatrix, modelViewTransform);					
			
			Matrix4.multiply(lightProjectionMatrix, lightModelViewMatrix, viewProjectionTransform);
			Matrix4.multiply(viewProjectionTransform, modelTransform, modelViewProjectionTransform);
			
			// copy the value back into the lightModelViewMatrix to be used by the second pass
			lightModelViewMatrix.set(modelViewProjectionTransform.matrix);
			
			// set uniforms
			int uMVPtr = getUniformLocation(Constants.UNIFORM_MV); 
			if( uMVPtr != -1 ){
				glUniformMatrix4fv(uMVPtr, 1, false, modelViewTransform.matrix, 0);
			}
					
			int uMVPPtr = getUniformLocation(Constants.UNIFORM_MVP); 
			if( uMVPPtr != -1 ){
				glUniformMatrix4fv(uMVPPtr, 1, false, modelViewProjectionTransform.matrix, 0);
			}			
						
		}		
		else if( pass == 1 ){
			currentProgram = super.bind(); 
			
			if( currentProgram >= 0 ){
				
				// send the shadow projection matrix
				int uShadowProjMatPtr = getUniformLocation(Constants.UNIFORM_SHADOW_PROJ_MATRIX); 
				if( uShadowProjMatPtr != -1 ){
					glUniformMatrix4fv(uShadowProjMatPtr, 1, false, lightModelViewMatrix.matrix, 0);
				}

				// send the depth texture
				int uTexPtr = getUniformLocation(Constants.UNIFORM_TEXTURE_SHADOW_MAP);				        
		        if( uTexPtr != -1 ){
		        	GLES20.glActiveTexture(GLES20.GL_TEXTURE1);	        
		        	GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, shadowMapTex.textureId);
		        	GLES20.glUniform1i(uTexPtr, 1);
		        }
		        
		        if( uShadowProjMatPtr > 0 && uTexPtr > 0 ){
		        	int uShadowEnabledPtr = getUniformLocation(Constants.UNIFORM_SHADOW_ENABLED); 
					if( uShadowEnabledPtr != -1 ){
						GLES20.glUniform1i(uShadowEnabledPtr, 1);
					}
		        } else{
		        	int uShadowEnabledPtr = getUniformLocation(Constants.UNIFORM_SHADOW_ENABLED); 
					if( uShadowEnabledPtr != -1 ){
						GLES20.glUniform1i(uShadowEnabledPtr, -1);
					}
		        }
			}
		}
		
		return currentProgram; 
	}
	
	@Override
	public void unbind(){
		super.unbind(); 
		
		usingDepthShader = false;
	}
	
	public int getAttribLocation(String attribute){
		if( usingDepthShader ){
			return depthShader.getAttribLocation(attribute);
		} else{
			return GLES20.glGetAttribLocation(program, attribute);
		}
	}
	
	public int getUniformLocation(String unifrom){
		if( usingDepthShader ){
			return depthShader.getUniformLocation(unifrom);
		} else{
			return GLES20.glGetUniformLocation(program, unifrom);
		}
	} 

}
