/**
 * SkinnedMaterial.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 25, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.material;

import static android.opengl.GLES20.glUniformMatrix4fv;
import android.content.Context;
import android.opengl.GLES20;

import com.wmp.nofrills3d.scene.Constants;

public class SkinnedMaterial extends Material{
	
	protected float[] boneTransforms = null; 
	protected int boneTransformsLength = 0; 
	protected int boneTransformsCount = 0; 
	
	/**
	 * @param context
	 * @param vertexShaderResId
	 * @param fragmentShaderResId
	 */
	public SkinnedMaterial(Context context, int vertexShaderResId,
			int fragmentShaderResId) {
		
		super(context, vertexShaderResId, fragmentShaderResId);
		
	}
	
	public void setBoneTransforms(float[] transforms){
		if( boneTransforms == null ){			
			boneTransformsLength = transforms.length;
			boneTransformsCount = boneTransformsLength / 16;  
			boneTransforms = new float[boneTransformsLength];			
		}
		
		for( int i=0; i<boneTransformsLength; i++ ){
			boneTransforms[i] = transforms[i]; 
		}
	}
	
	@Override
	public int bind(){
		int program = super.bind(); 
		
		if( program > 0 ){
			// bind bone poses
			if( boneTransforms != null ){
				int uBoneTransformsPtr = getUniformLocation(Constants.UNIFORM_BONE_TRANSFORMSN); 
				if( uBoneTransformsPtr != -1 ){
					glUniformMatrix4fv(uBoneTransformsPtr, boneTransformsCount, false, boneTransforms, 0);
				}
			}
			
			int uBoneCountPtr = getUniformLocation(Constants.UNIFORM_BONE_COUNT); 
			if( uBoneCountPtr != -1 ){
				GLES20.glUniform1i(uBoneCountPtr, boneTransformsCount);
			}
		}
		
		return program; 
	}

}
