/**
 * Matrix.java (ColourExplosion)
 * Created by Joshua Newnham on Apr 9, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.math;

import java.util.StringTokenizer;

import android.opengl.Matrix;
import android.util.FloatMath;

import com.wmp.nofrills3d.utils.Logger;

public class Matrix4 {
	
	// column-major 
	// 0	4	8	12
	// 1	5	9	13
	// 2	6	10	14
	// 3	7	11	15
	
	public Vector3 getTranslationAsVector3() {
        return new Vector3(matrix[Matrix4.M14], matrix[Matrix4.M24], matrix[Matrix4.M34]);
    }

    public Vector3 getTranslationAsVector3(Vector3 vector) {
        vector.set(matrix[Matrix4.M14], matrix[Matrix4.M24], matrix[Matrix4.M34]);
        
        return vector; 
    }

    public Quaternion getRotationAsQuaternion() {        
        return Quaternion.createFromRotationMatrix(
        		matrix[Matrix4.M11], matrix[Matrix4.M12], matrix[Matrix4.M13], 
        		       matrix[Matrix4.M21], matrix[Matrix4.M22], matrix[Matrix4.M23], 
        		              matrix[Matrix4.M31], matrix[Matrix4.M32], matrix[Matrix4.M33]);
    }

    public Quaternion getRotationAsQuaternion(Quaternion q) {
        Quaternion res = Quaternion.createFromRotationMatrix(
        		matrix[Matrix4.M11], matrix[Matrix4.M12], matrix[Matrix4.M13], 
 		       matrix[Matrix4.M21], matrix[Matrix4.M22], matrix[Matrix4.M23], 
 		              matrix[Matrix4.M31], matrix[Matrix4.M32], matrix[Matrix4.M33]);
        
        q.set(res.getX(), res.getY(), res.getZ(), res.getW());
        
        return q; 
    }

	public final static int M11 = 0; 	public final static int M12 = 1; 	public final static int M13 = 2; 	public final static int M14 = 3;
	public final static int M21 = 4; 	public final static int M22 = 5; 	public final static int M23 = 6; 	public final static int M24 = 7;
	public final static int M31 = 8; 	public final static int M32 = 9; 	public final static int M33 = 10; 	public final static int M34 = 11;
	public final static int M41 = 12; 	public final static int M42 = 13; 	public final static int M43 = 14; 	public final static int M44 = 15;		
	
	public float[] matrix = new float[16]; 
	
	public Matrix4(){
		this.reset(); 
	}
	
	public Matrix4( 
			float m11, float m12, float m13, float m14, 
			float m21, float m22, float m23, float m24, 
			float m31, float m32, float m33, float m34, 
			float m41, float m42, float m43, float m44){
		this.matrix[M11] = m11;	this.matrix[M12] = m12;	this.matrix[M13] = m13;	this.matrix[M14] = m14;	 
		this.matrix[M21] = m21;	this.matrix[M22] = m22;	this.matrix[M23] = m23;	this.matrix[M24] = m24;
		this.matrix[M31] = m31;	this.matrix[M32] = m32;	this.matrix[M33] = m33;	this.matrix[M34] = m34;
		this.matrix[M41] = m41;	this.matrix[M42] = m42;	this.matrix[M43] = m43;	this.matrix[M44] = m44;
	}
	
	public static int getColumnMajorIndexFromRowMajorIndex(int rowmajorindex, int width, int height) {
	    return getRowMajorIndexFromColumnMajorIndex(rowmajorindex, height, width);
	}
	
	public static int getRowMajorIndexFromColumnMajorIndex(int columnmajorindex, int width, int height) {
	    int row = columnmajorindex % height;
	    int column = columnmajorindex / height;
	    return row * width + column;
	}
	
	public Matrix4(float[] matrix){
		this.set(matrix);
	}		
	
	public void set(float[] matrix){
		for( int i=0; i<16; i++ ){
			this.matrix[i] = matrix[i]; 
		}
	}
	
	public void set(String data){
		StringTokenizer parts = new StringTokenizer(data, " ");
		int numTokens = parts.countTokens();
		
		if( numTokens != 16 ){
			return; 
		}
		
		for( int i=0; i<numTokens; i++ ){
			String val = parts.nextToken(); 		
			this.matrix[i] = Float.parseFloat(val); 
		}		
	}
	
	public void setTranslation(float x, float y, float z){
		setM41(x); 
		setM42(y);
		setM43(z); 
	}
	
	public void reset(){
		Matrix.setIdentityM(this.matrix, 0);
	}		
	
	// Row 1 getter and setters
	
	public void setM11(float v){
		this.matrix[M11] = v; 
	}
	
	public float getM11(){
		return this.matrix[M11]; 
	}
	
	public void setM12(float v){
		this.matrix[M12] = v; 
	}
	
	public float getM12(){
		return this.matrix[M12]; 
	}
	
	public void setM13(float v){
		this.matrix[M13] = v; 
	}
	
	public float getM13(){
		return this.matrix[M13]; 
	}
	
	public void setM14(float v){
		this.matrix[M14] = v; 
	}
	
	public float getM14(){
		return this.matrix[M14]; 
	}
	
	// Row 2 getter and setters
	
	public void setM21(float v){
		this.matrix[M21] = v; 
	}
	
	public float getM21(){
		return this.matrix[M21]; 
	}
	
	public void setM22(float v){
		this.matrix[M22] = v; 
	}
	
	public float getM22(){
		return this.matrix[M22]; 
	}
	
	public void setM23(float v){
		this.matrix[M23] = v; 
	}
	
	public float getM23(){
		return this.matrix[M23]; 
	}
	
	public void setM24(float v){
		this.matrix[M24] = v; 
	}
	
	public float getM24(){
		return this.matrix[M24]; 
	}
	
	// Row 3 getter and setters
	
	public void setM31(float v){
		this.matrix[M31] = v; 
	}
	
	public float getM31(){
		return this.matrix[M31]; 
	}
	
	public void setM32(float v){
		this.matrix[M32] = v; 
	}
	
	public float getM32(){
		return this.matrix[M32]; 
	}
	
	public void setM33(float v){
		this.matrix[M33] = v; 
	}
	
	public float getM33(){
		return this.matrix[M33]; 
	}
	
	public void setM34(float v){
		this.matrix[M34] = v; 
	}
	
	public float getM34(){
		return this.matrix[M34]; 
	}
	
	// Row 4 getter and setters
	
	public void setM41(float v){
		this.matrix[M41] = v; 
	}
	
	public float getM41(){
		return this.matrix[M41]; 
	}
	
	public void setM42(float v){
		this.matrix[M42] = v; 
	}
	
	public float getM42(){
		return this.matrix[M42]; 
	}
	
	public void setM43(float v){
		this.matrix[M43] = v; 
	}
	
	public float getM43(){
		return this.matrix[M43]; 
	}
	
	public void setM44(float v){
		this.matrix[M44] = v; 
	}
	
	public float getM44(){
		return this.matrix[M44]; 
	}
	
	public Vector3 getUp()
    {
        return new Vector3(matrix[Matrix4.M21], matrix[Matrix4.M22], matrix[Matrix4.M23]);        
    }
	
	public void setUp(Vector3 value){
		matrix[Matrix4.M21] = value.x;
		matrix[Matrix4.M22] = value.y;
		matrix[Matrix4.M23] = value.z;
	}
	
	public Vector3 getRight()
    {
        return new Vector3(-matrix[Matrix4.M11], -matrix[Matrix4.M12], -matrix[Matrix4.M13]);        
    }
	
	public void setRight(Vector3 value){
		matrix[Matrix4.M11] = value.x;
		matrix[Matrix4.M12] = value.y;
		matrix[Matrix4.M13] = value.z;
	}
	
	public Vector3 getForward()
    {
        return new Vector3(matrix[Matrix4.M31], matrix[Matrix4.M32], matrix[Matrix4.M33]);                
    }
	
	public void setForward(Vector3 value){
		matrix[Matrix4.M31] = -value.x;
		matrix[Matrix4.M32] = -value.y;
		matrix[Matrix4.M33] = -value.z;
	}
	
	public Matrix4 transpose(){
		float[] tmpMat = new float[16];
		for( int i=0; i<16; i++ ){
			tmpMat[i] = matrix[i]; 
		}
		
		Matrix.transposeM(this.matrix,  0, tmpMat, 0);
		
		return this; 
	}
	
	public Matrix4 clone(){
		return new Matrix4(matrix);
	}	
	
	public Matrix4 transposeClone(){
		Matrix4 mat = new Matrix4();
		Matrix4.transpose(this, mat);
		return mat; 
	}
	
	public String toString(){
		StringBuffer sb = new StringBuffer();		
		
		sb.append("[");
		for( int r=0; r<4; r++ ){
			//sb.append("[");
			for( int c=0; c<4; c++ ){
				sb.append(matrix[r * 4 + c]);
				if( c < 4-1 ){
					sb.append(", ");
				}
			}
			if( r < 4-1 ){
				sb.append(";");
			}
		}
		sb.append("]");
		
		return sb.toString();
	}

	public static Matrix4 createIdentity(){
		Matrix4 mat = new Matrix4(); 
		Matrix.setIdentityM(mat.matrix, 0);
		return mat; 
	}	
	
	public static Matrix4 createLookAt(Vector3 eyePosition, Vector3 centre, Vector3 up){
		Matrix4 result = new Matrix4(); 
		
		Matrix.setLookAtM(result.matrix, 
				0, 
				eyePosition.x, eyePosition.y, eyePosition.z, 
				centre.x, centre.y, centre.z, 
				up.x, up.y, up.z);
		
		return result; 
	}		
	
	public static Matrix4 createOrthographic(float left, float right, float bottom, float top, float near, float far){
		Matrix4 result = new Matrix4();		
		Matrix.orthoM(result.matrix, 0, left, right, bottom, top, near, far);		
		return result; 
	}
	
	public static Matrix4 createPerspective(float yFovInDegrees, float aspect, float near, float far){
		Matrix4 result = new Matrix4(); 
        Matrix.perspectiveM(result.matrix, 0, yFovInDegrees, aspect, near, far);
						
		return result; 
	}
	
	public static Matrix4 createTranslation(float xPosition, float yPosition, float zPosition)
    {
        Matrix4 result = createIdentity(); 
        Matrix.translateM(result.matrix, 0, xPosition, yPosition, zPosition);
		
		return result;
    }
	
	public static Matrix4 createScale(float xScale, float yScale, float zScale){
		Matrix4 result = createIdentity(); 		
		Matrix.scaleM(result.matrix, 0, xScale, yScale, zScale);
		
		return result; 
	}	
	
	public static Matrix4 createFromYawPitchRoll(float yaw, float pitch, float roll)
	{
	    Quaternion quaternion = Quaternion.createFromYawPitchRoll(yaw, pitch, roll);	    
	    return createFromQuaternion(quaternion);
	}
	
	public static Matrix4 createFromEulerAngles(float x, float y, float z)
	{
		Matrix4 result = Matrix4.createIdentity(); 
		Matrix.setRotateEulerM(result.matrix, 0, x, y, z);
		return result; 
	}
	
//	public static Matrix4 createFromQuaternion(Quaternion quaternion)
//    {
//		return createFromQuaternion(quaternion.getX(), quaternion.getY(), quaternion.getZ(), quaternion.getW());
//    }
//	
//	public static Matrix4 createFromQuaternion(float x, float y, float z, float w)
//    {
//		Matrix4 matrix = new Matrix4();	    
//	    
//	    float x2 = x * x;
//		float y2 = y * y;
//		float z2 = z * z;
//		float xy = x * y;
//		float xz = x * z;
//		float yz = y * z;
//		float wx = w * x;
//		float wy = w * y;
//		float wz = w * z;
//
//		matrix.matrix[0] = 1.0f - 2.0f * (y2 + z2);
//		matrix.matrix[1] = 2.0f * (xy - wz);
//		matrix.matrix[2] = 2.0f * (xz + wy);
//		matrix.matrix[3] = 0;
//
//		matrix.matrix[4] = 2.0f * (xy + wz);
//		matrix.matrix[5] = 1.0f - 2.0f * (x2 + z2);
//		matrix.matrix[6] = 2.0f * (yz - wx);
//		matrix.matrix[7] = 0;
//
//		matrix.matrix[8] = 2.0f * (xz - wy);
//		matrix.matrix[9] = 2.0f * (yz + wx);
//		matrix.matrix[10] = 1.0f - 2.0f * (x2 + y2);
//		matrix.matrix[11] = 0;
//
//		matrix.matrix[12] = 0;
//		matrix.matrix[13] = 0;
//		matrix.matrix[14] = 0;
//		matrix.matrix[15] = 1;
//	    
//	    return matrix;
//    }
	
	public static Matrix4 createFromTranslationAndQuaternion(float tx, float ty, float tz, float ox, float oy, float oz, float ow){
		Matrix4 translation = createTranslation(tx, ty, tz);
		
		Matrix4 rotation = Matrix4.createIdentity(); 
		Matrix.setRotateEulerM(rotation.matrix, 0, ox, oy, oz);
		
		Matrix4 res = Matrix4.createIdentity(); 

		Matrix4.multiply(translation, rotation, res);
		
		return res; 
	}
	
	public static Matrix4 createFromQuaternion(Quaternion quaternion)
    {
		Matrix4 result = new Matrix4(); 
		return createFromQuaternion(quaternion, result); 
    }
	
	public static Matrix4 createFromQuaternion(Quaternion quaternion, Matrix4 result)
    {
	    float num9 = quaternion.quaternion[Quaternion.X] * quaternion.quaternion[Quaternion.X];
	    float num8 = quaternion.quaternion[Quaternion.Y] * quaternion.quaternion[Quaternion.Y];
	    float num7 = quaternion.quaternion[Quaternion.Z] * quaternion.quaternion[Quaternion.Z];
	    float num6 = quaternion.quaternion[Quaternion.X] * quaternion.quaternion[Quaternion.Y];
	    float num5 = quaternion.quaternion[Quaternion.Z] * quaternion.quaternion[Quaternion.W];
	    float num4 = quaternion.quaternion[Quaternion.Z] * quaternion.quaternion[Quaternion.X];
	    float num3 = quaternion.quaternion[Quaternion.Y] * quaternion.quaternion[Quaternion.W];
	    float num2 = quaternion.quaternion[Quaternion.Y] * quaternion.quaternion[Quaternion.Z];
	    float num = quaternion.quaternion[Quaternion.X] * quaternion.quaternion[Quaternion.W];
	    result.matrix[M11] = 1f - (2f * (num8 + num7));
	    result.matrix[M12] = 2f * (num6 + num5);
	    result.matrix[M13] = 2f * (num4 - num3);
	    result.matrix[M14] = 0f;
	    result.matrix[M21] = 2f * (num6 - num5);
	    result.matrix[M22] = 1f - (2f * (num7 + num9));
	    result.matrix[M23] = 2f * (num2 + num);
	    result.matrix[M24] = 0f;
	    result.matrix[M31] = 2f * (num4 + num3);
	    result.matrix[M32] = 2f * (num2 - num);
	    result.matrix[M33] = 1f - (2f * (num8 + num9));
	    result.matrix[M34] = 0f;
	    result.matrix[M41] = 0f;
	    result.matrix[M42] = 0f;
	    result.matrix[M43] = 0f;
	    result.matrix[M44] = 1f;
	    
	    return result;
    }
	
	public static Matrix4 createFromAxisAngle(Vector3 axis, float angle)
    {
        Matrix4 matrix = new Matrix4();
	    return createFromAxisAngle(axis, angle, matrix);

    }
	
	public static Matrix4 createFromAxisAngle(Vector3 axis, float angle, Matrix4 matrix)
    {
	    float x = axis.x;
	    float y = axis.y;
	    float z = axis.z;
	    float num2 = (float) Math.sin((double) angle);
	    float num = (float) Math.cos((double) angle);
	    float num11 = x * x;
	    float num10 = y * y;
	    float num9 = z * z;
	    float num8 = x * y;
	    float num7 = x * z;
	    float num6 = y * z;
	    matrix.setM11(num11 + (num * (1f - num11)));
	    matrix.setM12((num8 - (num * num8)) + (num2 * z));
	    matrix.setM13((num7 - (num * num7)) - (num2 * y));
	    matrix.setM14(0f);
	    matrix.setM21((num8 - (num * num8)) - (num2 * z));
	    matrix.setM22(num10 + (num * (1f - num10)));
	    matrix.setM23((num6 - (num * num6)) + (num2 * x));
	    matrix.setM24(0f);
	    matrix.setM31((num7 - (num * num7)) + (num2 * y));
	    matrix.setM32((num6 - (num * num6)) - (num2 * x));
	    matrix.setM33(num9 + (num * (1f - num9)));
	    matrix.setM34(0f);
	    matrix.setM41(0f);
	    matrix.setM42(0f);
	    matrix.setM43(0f);
	    matrix.setM44(1f);
	    
	    return matrix;

    }		
		
	public static Matrix4 multiply(Matrix4 matrix1, Matrix4 matrix2, Matrix4 result){
		Matrix.multiplyMM(result.matrix, 0, matrix1.matrix, 0, matrix2.matrix, 0);
		return result; 
    }
	
	public static Matrix4 invert(Matrix4 matrix, Matrix4 result){
		if( !Matrix.invertM(result.matrix, 0, matrix.matrix, 0) ){
			if( Logger.Logging ){
				Logger.LogWarning("Matrix4", "Failed to invert matrix");
			}
		}	
		
		return result; 
	}
	
	public static Matrix4 transpose(Matrix4 normalMatrix, Matrix4 result){
		Matrix.transposeM(result.matrix, 0, normalMatrix.matrix, 0);
		return result; 
	}
	
	public static Matrix4 lerp(Matrix4 matrix1, Matrix4 matrix2, float t)
    {
		Matrix4 result = new Matrix4();
		
		return Matrix4.lerp(matrix1, matrix2, t, result); 
    }
	
	public static Matrix4 lerp(Matrix4 matrix1, Matrix4 matrix2, float t, Matrix4 result)
    {
		for( int i=0; i<matrix1.matrix.length; i++ ){
			result.matrix[i] = matrix1.matrix[i] + ((matrix2.matrix[i] - matrix1.matrix[i]) * t);  
		}
		return result; 
    }
	
	public static Matrix4 blend(Matrix4 fromTransform, Matrix4 toTransform, float blend, Matrix4 result)
    {		
        Vector3 vt1 = new Vector3(); Vector3 vs1 = new Vector3(); Quaternion q1 = new Quaternion();
        Matrix4.decompose(fromTransform, vt1, q1, vs1);

        Vector3 vt2 = new Vector3(); Vector3 vs2 = new Vector3(); Quaternion q2 = new Quaternion();
        Matrix4.decompose(toTransform, vt2, q2, vs2);

        Vector3 vtBlend = Vector3.lerp(vt1, vt2, blend);
        Vector3 vsBlend = Vector3.lerp(vs1, vs2, blend);
        Quaternion qBlend = Quaternion.lerp(q1, q2, blend);
        
        Matrix4 translation = createTranslation(vtBlend.x, vtBlend.y, vtBlend.z);        
        Matrix4 rotation = createFromQuaternion(qBlend);     
        
        //Matrix4 rotation = createFromEulerAngles(qBlend.x, qBlend.y, qBlend.z);
        Matrix4 scale = createScale(vsBlend.x, vsBlend.y, vsBlend.z);        
		
		result.reset(); 
		Matrix4.multiply(Matrix4.multiply(translation, rotation, result), scale, result);		        
            
        return result; 
    }   
	
	public static void decompose(Matrix4 matrix, Vector3 position, Quaternion rotation, Vector3 scale){
		decomposeScale(matrix, scale);
		decomposeRotation(matrix, rotation, scale);
		decomposeTranslation(matrix, position); 		
	}
	
	public static void decompose(Matrix4 matrix, Vector3 position, Vector3 rotation, Vector3 scale){
		decomposeScale(matrix, scale);
		decomposeRotation(matrix, rotation, scale);
		decomposeTranslation(matrix, position); 		
	}
	
	public static Vector3 decomposeScale(Matrix4 matrix, Vector3 scale){
		scale.x = (float)Math.sqrt(matrix.getM11() * matrix.getM11() + matrix.getM12() * matrix.getM12() + matrix.getM13() * matrix.getM13());
		scale.y = (float)Math.sqrt(matrix.getM21() * matrix.getM21() + matrix.getM22() * matrix.getM22() + matrix.getM23() * matrix.getM23());
		scale.z = (float)Math.sqrt(matrix.getM31() * matrix.getM31() + matrix.getM32() * matrix.getM32() + matrix.getM33() * matrix.getM33());
		
		return scale; 
	}
	
	public static Vector3 decomposeRotation(Matrix4 matrix, Vector3 rotation){
		return decomposeRotation(matrix, rotation, null); 
	}
	
	public static Vector3 decomposeRotation(Matrix4 matrix, Vector3 rotation, Vector3 scale){
		if( scale == null ){
			scale = Vector3.createZero();
			decomposeScale(matrix, scale);
		}
		
		Matrix4 rotMat = Matrix4.createIdentity(); 
		createRotationMatrixFromMatrix4(matrix, rotMat, scale);
				
		rotation.x = (float)Math.atan2(-rotMat.getM32(), rotMat.getM33());
		rotation.y = (float)Math.atan2(rotMat.getM31(), (float)Math.sqrt(rotMat.getM32()*rotMat.getM32() + rotMat.getM33()*rotMat.getM33()) );
		rotation.z = (float)Math.atan2(-rotMat.getM21(), rotMat.getM11());
		
		// convert radians to degrees 
		rotation.x = MathHelper.rad2Deg(rotation.x);
		rotation.y = MathHelper.rad2Deg(rotation.y);
		rotation.z = MathHelper.rad2Deg(rotation.z);
		
		return rotation; 
	}
	
	public static Quaternion decomposeRotation(Matrix4 matrix, Quaternion rotation, Vector3 scale){
		if( scale == null ){
			scale = Vector3.createZero();
			decomposeScale(matrix, scale);
		}
		
		Matrix4 rotMat = Matrix4.createIdentity(); 
		createRotationMatrixFromMatrix4(matrix, rotMat, scale);
				
		if( scale.x == 0.0f || scale.y == 0.0f || scale.z == 0.0f ){
			rotation.set(Quaternion.Identity); 
		}
		
		Quaternion.createFromRotationMatrix(rotMat, rotation);
		
		return rotation; 
	}
	
	public static Matrix4 createRotationMatrixFromMatrix4(Matrix4 mat, Matrix4 rot){
		return createRotationMatrixFromMatrix4(mat, rot, null);
	}
	
	public static Matrix4 createRotationMatrixFromMatrix4(Matrix4 mat, Matrix4 rot, Vector3 scale){
		if( scale == null ){
			scale = Vector3.createZero();
			decomposeScale(mat, scale);
		}
		
		rot.setM11(mat.getM11()/scale.x);
		rot.setM12(mat.getM12()/scale.x);
		rot.setM13(mat.getM13()/scale.x);
		rot.setM14(0);
	    
		rot.setM21(mat.getM21()/scale.y);
		rot.setM22(mat.getM22()/scale.y);
		rot.setM23(mat.getM23()/scale.y);
		rot.setM24(0);
	    
		rot.setM31(mat.getM31()/scale.z);
		rot.setM32(mat.getM32()/scale.z);
		rot.setM33(mat.getM33()/scale.z);
		rot.setM34(0);

		rot.setM41(0);
		rot.setM42(0);
		rot.setM43(0);
		rot.setM44(1);
		
		return rot; 
	}
	
	public static Vector3 decomposeTranslation(Matrix4 mat){
		Vector3 translation = Vector3.createZero(); 
		
		return decomposeTranslation(mat, translation); 
	}
	
	public static Vector3 decomposeTranslation(Matrix4 mat, Vector3 translation){
		translation.x = mat.getM41(); 
		translation.y = mat.getM42();
		translation.z = mat.getM43(); 
		
		return translation; 
	}
}
