/**
 * Quaternion.java (ColourExplosion)
 * Created by Joshua Newnham on Apr 9, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.math;

import android.annotation.SuppressLint;



@SuppressLint("DefaultLocale")
public class Quaternion {
	
	public final static int X = 0;	
	public static final int Y = 1; 
	public static final int Z = 2; 
	public static final int W = 3; 
	
	public static final float[] Identity = new float[]{0,0,0,1}; 
	
	public float[] quaternion = new float[4];  
	
	public static Quaternion createIdentity(){
		return new Quaternion(0, 0, 0, 1);
	}
	
	public static Quaternion createFromAxisAngle(Vector3 axis, float angle)
    {
		Quaternion result = Quaternion.createIdentity();
		
        float num2 = angle * 0.5f;
	    float num = (float) Math.sin((double) num2);
	    float num3 = (float) Math.cos((double) num2);
	    result.quaternion[X] = axis.x * num;
	    result.quaternion[Y] = axis.y * num;
	    result.quaternion[Z] = axis.z * num;
	    result.quaternion[W] = num3;

	    return result; 
    }
	
	public static Quaternion createFromAxisAngle(Vector3 axis, float angle, Quaternion result)
    {
        float num2 = angle * 0.5f;
	    float num = (float) Math.sin((double) num2);
	    float num3 = (float) Math.cos((double) num2);
	    result.quaternion[X] = axis.x * num;
	    result.quaternion[Y] = axis.y * num;
	    result.quaternion[Z] = axis.z * num;
	    result.quaternion[W] = num3;

	    return result; 
    }
	
	public static Quaternion createFromYawPitchRoll(float yaw, float pitch, float roll)
	{
	    Quaternion quaternion = new Quaternion();
	    
	    float num9 = roll * 0.5f;
	    float num6 = (float) Math.sin((double) num9);
	    float num5 = (float) Math.cos((double) num9);
	    float num8 = pitch * 0.5f;
	    float num4 = (float) Math.sin((double) num8);
	    float num3 = (float) Math.cos((double) num8);
	    float num7 = yaw * 0.5f;
	    float num2 = (float) Math.sin((double) num7);
	    float num = (float) Math.cos((double) num7);
	    
	    quaternion.setX(((num * num4) * num5) + ((num2 * num3) * num6));
	    quaternion.setY(((num2 * num3) * num5) - ((num * num4) * num6));
	    quaternion.setZ(((num * num3) * num6) - ((num2 * num4) * num5));
	    quaternion.setW(((num * num3) * num5) + ((num2 * num4) * num6));
	    
	    return quaternion;
	}
	
	public static Quaternion createFromRotationMatrix(Matrix4 matrix, Quaternion result)
    {
        float num8 = (matrix.matrix[Matrix4.M11] + matrix.matrix[Matrix4.M22]) + matrix.matrix[Matrix4.M33];
	    if (num8 > 0f)
	    {
	        float num = (float) Math.sqrt((double) (num8 + 1f));
	        result.quaternion[Quaternion.W] = num * 0.5f;
	        num = 0.5f / num;
	        result.quaternion[Quaternion.X] = (matrix.matrix[Matrix4.M23] - matrix.matrix[Matrix4.M32]) * num;
	        result.quaternion[Quaternion.Y] = (matrix.matrix[Matrix4.M31] - matrix.matrix[Matrix4.M13]) * num;
	        result.quaternion[Quaternion.Z] = (matrix.matrix[Matrix4.M12] - matrix.matrix[Matrix4.M21]) * num;
	    }
	    else if ((matrix.matrix[Matrix4.M11] >= matrix.matrix[Matrix4.M22]) && (matrix.matrix[Matrix4.M11] >= matrix.matrix[Matrix4.M33]))
	    {
	        float num7 = (float) Math.sqrt((double) (((1f + matrix.matrix[Matrix4.M11]) - matrix.matrix[Matrix4.M22]) - matrix.matrix[Matrix4.M33]));
	        float num4 = 0.5f / num7;
	        result.quaternion[Quaternion.X] = 0.5f * num7;
	        result.quaternion[Quaternion.Y] = (matrix.matrix[Matrix4.M12] + matrix.matrix[Matrix4.M21]) * num4;
	        result.quaternion[Quaternion.Z] = (matrix.matrix[Matrix4.M13] + matrix.matrix[Matrix4.M31]) * num4;
	        result.quaternion[Quaternion.W] = (matrix.matrix[Matrix4.M23] - matrix.matrix[Matrix4.M32]) * num4;
	    }
	    else if (matrix.matrix[Matrix4.M22] > matrix.matrix[Matrix4.M33])
	    {
	        float num6 = (float) Math.sqrt((double) (((1f + matrix.matrix[Matrix4.M22]) - matrix.matrix[Matrix4.M11]) - matrix.matrix[Matrix4.M33]));
	        float num3 = 0.5f / num6;
	        result.quaternion[Quaternion.X] = (matrix.matrix[Matrix4.M21] + matrix.matrix[Matrix4.M12]) * num3;
	        result.quaternion[Quaternion.Y] = 0.5f * num6;
	        result.quaternion[Quaternion.Z] = (matrix.matrix[Matrix4.M32] + matrix.matrix[Matrix4.M23]) * num3;
	        result.quaternion[Quaternion.W] = (matrix.matrix[Matrix4.M31] - matrix.matrix[Matrix4.M13]) * num3;
	    }
	    else
	    {
	        float num5 = (float) Math.sqrt((double) (((1f + matrix.matrix[Matrix4.M33]) - matrix.matrix[Matrix4.M11]) - matrix.matrix[Matrix4.M22]));
	        float num2 = 0.5f / num5;
	        result.quaternion[Quaternion.X] = (matrix.matrix[Matrix4.M31] + matrix.matrix[Matrix4.M13]) * num2;
	        result.quaternion[Quaternion.Y] = (matrix.matrix[Matrix4.M32] + matrix.matrix[Matrix4.M23]) * num2;
	        result.quaternion[Quaternion.Z] = 0.5f * num5;
	        result.quaternion[Quaternion.W] = (matrix.matrix[Matrix4.M12] - matrix.matrix[Matrix4.M21]) * num2;
	    }

	    return result; 
    }
	
	public static Quaternion createfromAxes(Vector3 xAxis, Vector3 yAxis, Vector3 zAxis) {
        return createFromRotationMatrix(xAxis.x, yAxis.x, zAxis.x, 
        		xAxis.y, yAxis.y, zAxis.y,
                	xAxis.z, yAxis.z, zAxis.z);
    }
	
	/**
	 * https://code.google.com/p/jmonkeyengine/source/browse/trunk/engine/src/core/com/jme3/math/Quaternion.java?r=7672
	 */
	public static Quaternion createFromRotationMatrix(
			float m11, float m12, float m13,
            float m21, float m22, float m23,
            float m31, float m32, float m33) {
       
        float t = m11 + m22 + m33;
        
        float x, y, z, w;                 

        // we protect the division by s by ensuring that s>=1
        if (t >= 0) { // |w| >= .5
            float s = (float)Math.sqrt(t + 1); // |s|>=1 ...
            w = 0.5f * s;
            s = 0.5f / s;                 // so this division isn't bad
            x = (m32 - m23) * s;
            y = (m13 - m31) * s;
            z = (m21 - m12) * s;
        } else if ((m11 > m22) && (m11 > m33)) {
            float s = (float)Math.sqrt(1.0f + m11 - m22 - m33); // |s|>=1
            x = s * 0.5f; // |x| >= .5
            s = 0.5f / s;
            y = (m21 + m12) * s;
            z = (m13 + m31) * s;
            w = (m32 - m23) * s;
        } else if (m22 > m33) {
            float s = (float)Math.sqrt(1.0f + m22 - m11 - m33); // |s|>=1
            y = s * 0.5f; // |y| >= .5
            s = 0.5f / s;
            x = (m21 + m12) * s;
            z = (m32 + m23) * s;
            w = (m13 - m31) * s;
        } else {
            float s = (float)Math.sqrt(1.0f + m33 - m11 - m22); // |s|>=1
            z = s * 0.5f; // |z| >= .5
            s = 0.5f / s;
            x = (m13 + m31) * s;
            y = (m32 + m23) * s;
            w = (m21 - m12) * s;
        }

        return new Quaternion(x, y, z, w);
    }
	
	public static void divide(Quaternion quaternion1, Quaternion quaternion2, Quaternion result)
    {
        float x = quaternion1.quaternion[X];
	    float y = quaternion1.quaternion[Y];
	    float z = quaternion1.quaternion[Z];
	    float w = quaternion1.quaternion[W];
	    float num14 = (((quaternion2.quaternion[X] * quaternion2.quaternion[X]) 
	    			+ (quaternion2.quaternion[Y] * quaternion2.quaternion[Y])) 
	    				+ (quaternion2.quaternion[Z] * quaternion2.quaternion[Z])) 
	    					+ (quaternion2.quaternion[W] * quaternion2.quaternion[W]);
	    float num5 = 1f / num14;
	    float num4 = -quaternion2.quaternion[X] * num5;
	    float num3 = -quaternion2.quaternion[Y] * num5;
	    float num2 = -quaternion2.quaternion[Z] * num5;
	    float num = quaternion2.quaternion[W] * num5;
	    float num13 = (y * num2) - (z * num3);
	    float num12 = (z * num4) - (x * num2);
	    float num11 = (x * num3) - (y * num4);
	    float num10 = ((x * num4) + (y * num3)) + (z * num2);
	    result.setX( ((x * num) + (num4 * w)) + num13 );
	    result.setY( ((y * num) + (num3 * w)) + num12 );
	    result.setZ( ((z * num) + (num2 * w)) + num11 );
	    result.setW( (w * num) - num10 );

    }
	
	public static void dot(Quaternion quaternion1, Quaternion quaternion2, float result)
    {
        result = (((quaternion1.quaternion[0] * quaternion2.quaternion[0]) 
        		+ (quaternion1.quaternion[1] * quaternion2.quaternion[1])) 
        			+ (quaternion1.quaternion[2] * quaternion2.quaternion[2])) 
        				+ (quaternion1.quaternion[3] * quaternion2.quaternion[3]);
    }
	
	public static void inverse(Quaternion quaternion, Quaternion result)
    {
        float num2 = (((quaternion.quaternion[X] * quaternion.quaternion[X]) 
        			+ (quaternion.quaternion[Y] * quaternion.quaternion[Y])) 
        				+ (quaternion.quaternion[Z] * quaternion.quaternion[Z])) 
        					+ (quaternion.quaternion[W] * quaternion.quaternion[W]);
        
	    float num = 1f / num2;
	    
	    result.quaternion[X] = -quaternion.quaternion[X] * num;
	    result.quaternion[Y] = -quaternion.quaternion[Y] * num;
	    result.quaternion[Z] = -quaternion.quaternion[Z] * num;
	    result.quaternion[W] = quaternion.quaternion[W] * num;
    } 
	
	public static Quaternion lerp(Quaternion quaternion1, Quaternion quaternion2, float amount){
		Quaternion result = new Quaternion(); 
		return Quaternion.lerp(quaternion1, quaternion2, amount, result);
	}
	
	public static Quaternion lerp(Quaternion quaternion1, Quaternion quaternion2, float amount, Quaternion result)
    {
        float num = amount;
	    float num2 = 1f - num;
	    float num5 = (((quaternion1.quaternion[X] * quaternion2.quaternion[X]) 
	    		+ (quaternion1.quaternion[Y] * quaternion2.quaternion[Y])) 
	    			+ (quaternion1.quaternion[Z] * quaternion2.quaternion[Z])) 
	    				+ (quaternion1.quaternion[W] * quaternion2.quaternion[W]);
	    if (num5 >= 0f)
	    {
	        result.quaternion[X] = (num2 * quaternion1.quaternion[X]) + (num * quaternion2.quaternion[X]);
	        result.quaternion[Y] = (num2 * quaternion1.quaternion[Y]) + (num * quaternion2.quaternion[Y]);
	        result.quaternion[Z] = (num2 * quaternion1.quaternion[Z]) + (num * quaternion2.quaternion[Z]);
	        result.quaternion[W] = (num2 * quaternion1.quaternion[W]) + (num * quaternion2.quaternion[W]);
	    }
	    else
	    {
	        result.quaternion[X] = (num2 * quaternion1.quaternion[X]) - (num * quaternion2.quaternion[X]);
	        result.quaternion[Y] = (num2 * quaternion1.quaternion[Y]) - (num * quaternion2.quaternion[Y]);
	        result.quaternion[Z] = (num2 * quaternion1.quaternion[Z]) - (num * quaternion2.quaternion[Z]);
	        result.quaternion[W] = (num2 * quaternion1.quaternion[W]) - (num * quaternion2.quaternion[W]);
	    }
	    float num4 = (((result.quaternion[X] * result.quaternion[X]) + (result.quaternion[Y] * result.quaternion[Y])) 
	    			+ (result.quaternion[Z] * result.quaternion[Z])) + (result.quaternion[W] * result.quaternion[W]);
	    float num3 = 1f / ((float) Math.sqrt((double) num4));
	    
	    result.quaternion[X] *= num3;
	    result.quaternion[Y] *= num3;
	    result.quaternion[Z] *= num3;
	    result.quaternion[W] *= num3;
	    
	    return result; 
    }
	
	public static void slerp(Quaternion quaternion1, Quaternion quaternion2, float amount, Quaternion result)
    {
        float num2;
	    float num3;
	    float num = amount;
	    float num4 = (((quaternion1.quaternion[X] * quaternion2.quaternion[X]) 
	    			+ (quaternion1.quaternion[Y] * quaternion2.quaternion[Y])) 
	    				+ (quaternion1.quaternion[Z] * quaternion2.quaternion[Z])) 
	    					+ (quaternion1.quaternion[W] * quaternion2.quaternion[W]);
	    boolean flag = false;
	    if (num4 < 0f)
	    {
	        flag = true;
	        num4 = -num4;
	    }
	    if (num4 > 0.999999f)
	    {
	        num3 = 1f - num;
	        num2 = flag ? -num : num;
	    }
	    else
	    {
	        float num5 = (float) Math.acos((double) num4);
	        float num6 = (float) (1.0 / Math.sin((double) num5));
	        num3 = ((float) Math.sin((double) ((1f - num) * num5))) * num6;
	        num2 = flag ? (((float) -Math.sin((double) (num * num5))) * num6) : (((float) Math.sin((double) (num * num5))) * num6);
	    }
	    result.quaternion[X] = (num3 * quaternion1.quaternion[X]) + (num2 * quaternion2.quaternion[X]);
	    result.quaternion[Y] = (num3 * quaternion1.quaternion[Y]) + (num2 * quaternion2.quaternion[Y]);
	    result.quaternion[Z] = (num3 * quaternion1.quaternion[Z]) + (num2 * quaternion2.quaternion[Z]);
	    result.quaternion[W] = (num3 * quaternion1.quaternion[W]) + (num2 * quaternion2.quaternion[W]);
    }
	
	public static void scale(Quaternion quaternion1, float scaleFactor, Quaternion result)
    {
        result.quaternion[X] = quaternion1.quaternion[X] * scaleFactor;
	    result.quaternion[Y] = quaternion1.quaternion[Y] * scaleFactor;
	    result.quaternion[Z] = quaternion1.quaternion[Z] * scaleFactor;
	    result.quaternion[W] = quaternion1.quaternion[W] * scaleFactor;
    }


    public static void multiply(Quaternion quaternion1, Quaternion quaternion2, Quaternion result)
    {
        float x = quaternion1.quaternion[X];
	    float y = quaternion1.quaternion[Y];
	    float z = quaternion1.quaternion[Z];
	    float w = quaternion1.quaternion[W];
	    float num4 = quaternion2.quaternion[X];
	    float num3 = quaternion2.quaternion[Y];
	    float num2 = quaternion2.quaternion[Z];
	    float num = quaternion2.quaternion[W];
	    float num12 = (y * num2) - (z * num3);
	    float num11 = (z * num4) - (x * num2);
	    float num10 = (x * num3) - (y * num4);
	    float num9 = ((x * num4) + (y * num3)) + (z * num2);
	    result.quaternion[X] = ((x * num) + (num4 * w)) + num12;
	    result.quaternion[Y] = ((y * num) + (num3 * w)) + num11;
	    result.quaternion[Z] = ((z * num) + (num2 * w)) + num10;
	    result.quaternion[W] = (w * num) - num9;
    }
    
    public static void multiply(Quaternion quaternion, Vector3 vector, Vector3 result)
    {

    	Vector3 tmp3Vec; 
    	Vector3 tmp1Vec = new Vector3();
    	Vector3 tmp2Vec = new Vector3();
    	
    	tmp3Vec = new Vector3(quaternion.quaternion[X], quaternion.quaternion[Y], quaternion.quaternion[Z]);
    	Vector3.crossProduct(tmp3Vec, vector, tmp1Vec);
    	Vector3.crossProduct(tmp3Vec, tmp1Vec, tmp2Vec);
    	
    	tmp1Vec.scale(2.0f * quaternion.quaternion[W]);
    	tmp2Vec.scale(2.0f);
    	
    	tmp1Vec.add(tmp2Vec);
    	tmp1Vec.add(vector);
    	
    	result.set(tmp1Vec.x, tmp1Vec.y, tmp1Vec.z);
    }       
    
    public static void normalize(Quaternion quaternion, Quaternion result)
    {
        float num2 = (((quaternion.quaternion[X] * quaternion.quaternion[X]) 
        			+ (quaternion.quaternion[Y] * quaternion.quaternion[Y])) 
        				+ (quaternion.quaternion[Z] * quaternion.quaternion[Z])) 
        					+ (quaternion.quaternion[W] * quaternion.quaternion[W]);
	    
        float num = 1f / ((float) Math.sqrt((double) num2));
	    
	    result.quaternion[X] = quaternion.quaternion[X] * num;
	    result.quaternion[Y] = quaternion.quaternion[Y] * num;
	    result.quaternion[Z] = quaternion.quaternion[Z] * num;
	    result.quaternion[W] = quaternion.quaternion[W] * num;
    }
	
	public static float computeW(float x, float y, float z){
		float w = 0.0f; 
		
		float t = 1.0f - ( x * x ) - ( y * y ) - ( z * z );
	    if ( t < 0.0f )
	    {
	        w = 0.0f;
	    }
	    else
	    {
	        w = -(float)Math.sqrt(t);
	    }	
	    
	    return w;
	}
	
	public Quaternion(){
		quaternion[X] = 0; quaternion[Y] = 0; quaternion[Z] = 0; quaternion[W] = 1;
	}
	
	public Quaternion(float x, float y, float z){
		quaternion[X] = x; quaternion[Y] = y; quaternion[Z] = z;
		computeW(); 
	}
	
	public Quaternion(float x, float y, float z, float w){
		quaternion[X] = x; quaternion[Y] = y; quaternion[Z] = z; quaternion[W] = w;
	}
	
	public void set(float[] xyzw){
		quaternion[X] = xyzw[0];
		quaternion[Y] = xyzw[1];
		quaternion[Z] = xyzw[2];
		quaternion[W] = xyzw[3];
	}
	
	public void set(float x, float y, float z){
		quaternion[X] = x; quaternion[Y] = y; quaternion[Z] = z; 
		computeW(); 
	}
	
	public void set(float x, float y, float z, float w){
		quaternion[X] = x; quaternion[Y] = y; quaternion[Z] = z; quaternion[W] = w;
	}
	
	public float getX(){
		return quaternion[X];
	}
	
	public void setX(float v){
		quaternion[X] = v; 
	}
	
	public float getY(){
		return quaternion[Y];
	}
	
	public void setY(float v){
		quaternion[Y] = v; 
	}
	
	public float getZ(){
		return quaternion[Z];
	}
	
	public void setZ(float v){
		quaternion[Z] = v; 
	}
	
	public float getW(){
		return quaternion[W];
	}
	
	public void setW(float v){
		quaternion[W] = v; 
	}
	
	public Quaternion toDegrees(){
		quaternion[X] = MathHelper.rad2Deg(quaternion[X]);
		quaternion[Y] = MathHelper.rad2Deg(quaternion[Y]);
		quaternion[Z] = MathHelper.rad2Deg(quaternion[Z]);
		quaternion[W] = MathHelper.rad2Deg(quaternion[W]);
		
		return this; 
	}
	
	public Quaternion toRadians(){
		quaternion[X] = MathHelper.deg2Rad(quaternion[X]);
		quaternion[Y] = MathHelper.deg2Rad(quaternion[Y]);
		quaternion[Z] = MathHelper.deg2Rad(quaternion[Z]);
		quaternion[W] = MathHelper.deg2Rad(quaternion[W]);
		
		return this; 
	}
	
	public void computeW(){
		float x = quaternion[X];
		float y = quaternion[Y];
		float z = quaternion[Z];
		float w = 0.0f; 
		
		float t = 1.0f - ( x * x ) - ( y * y ) - ( z * z );
	    if ( t < 0.0f )
	    {
	        w = 0.0f;
	    }
	    else
	    {
	        w = -(float)Math.sqrt(t);
	    }	
	    
	    quaternion[W] = w;
	}
	
	public void conjugate()
	{
		this.quaternion[X] = -this.quaternion[X];
		this.quaternion[Y] = -this.quaternion[Y];
		this.quaternion[Z] = -this.quaternion[Z];
	}
	
	public float length()
    {        
		float num = lengthSquared();
		return (float) Math.sqrt((double) num);
    }


    public float lengthSquared()
    {
        return  (((this.quaternion[X] * this.quaternion[X]) + (this.quaternion[Y] * this.quaternion[Y])) + (this.quaternion[Z] * this.quaternion[Z])) 
    	+ (this.quaternion[W] * this.quaternion[W]);
    }
    
    /**
     * <code>toAngles</code> returns this quaternion converted to Euler
     * rotation angles (yaw,roll,pitch).<br/>
     * See http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToEuler/index.htm
     * Source: https://code.google.com/p/jmonkeyengine/source/browse/trunk/engine/src/core/com/jme3/math/Quaternion.java?r=7672
     */
    public Vector3 toEulerAngles() {
        return toEulerAngles(Vector3.createZero());
    }
    
    public Vector3 toEulerAngles(Vector3 eulerAngles) {
        float x = getX(); 
        float y = getY(); 
        float z = getZ(); 
        float w = getW(); 
        float sqw = w * w;
        float sqx = x * x;
        float sqy = y * y;
        float sqz = z * z;
        float unit = sqx + sqy + sqz + sqw; // if normalized is one, otherwise
        // is correction factor
        float test = x * y + z * w;
        if (test > 0.499 * unit) { // singularity at north pole
        	eulerAngles.y = 2 * (float)Math.atan2(x, w);
            eulerAngles.z = MathHelper.PI_OVER_2;
            eulerAngles.x = 0;
        } else if (test < -0.499 * unit) { // singularity at south pole
        	eulerAngles.y = -2 * (float)Math.atan2(x, w);
            eulerAngles.z = -MathHelper.PI_OVER_2;
            eulerAngles.x = 0;
        } else {
        	eulerAngles.y = (float)Math.atan2(2 * y * w - 2 * x * z, sqx - sqy - sqz + sqw); // roll or heading 
            eulerAngles.z = (float)Math.asin(2 * test / unit); // pitch or attitude
            eulerAngles.x = (float)Math.atan2(2 * x * w - 2 * y * z, -sqx + sqy - sqz + sqw); // yaw or bank
        }
        
        return eulerAngles; 
    }
	
	public Quaternion clone(){
		return new Quaternion(quaternion[X], quaternion[Y], quaternion[Z], quaternion[W]);
	}
	
	@Override
	public String toString(){
		return String.format("Q(%f, %f, %f, %f)", quaternion[X], quaternion[Y], quaternion[Z], quaternion[W]);
	}
}
