/**
 * Ray.java (ColourExplosion)
 * Created by Joshua Newnham on Apr 9, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.math;

import com.wmp.nofrills3d.entities.Transform;
import com.wmp.nofrills3d.physics.BoxCollider;
import com.wmp.nofrills3d.physics.CollisionDetector;
import com.wmp.nofrills3d.physics.ContactCollection;
import com.wmp.nofrills3d.physics.ICollider;
import com.wmp.nofrills3d.physics.SphereCollider;
import com.wmp.nofrills3d.utils.Logger;

public class Ray implements ICollider {	
	
	public static final String TAG = "Ray";
	
	protected Vector3 point; 
	
	protected Vector3 vector;
	
	protected Transform transform; 
	
	public Ray(Vector3 point, Vector3 vector){
		this.point = point; 
		this.vector = vector; 
	}	
	
	@Override
	public void update(float elapsedTime) {
		// TODO Auto-generated method stub	
	}
	
	public float intersects(BoxCollider box){
		if( Logger.Logging ){
			Logger.LogTODO(TAG, "intersects(BoxCollider) incomplete");
		}
		return -1; 
	}
	
	public boolean intersects(SphereCollider sphere) {
        return distanceBetween(sphere.getCentre(), this) < sphere.radius;
    }        		
	
	/**
	 * http://mathworld.wolfram.com/Point-LineDistance3-Dimensional.html
	 * @param point
	 * @param ray
	 * @return
	 */
    public static float distanceBetween(Vector3 point, Ray ray) {
        Vector3 p1ToPoint = Vector3.subtract(ray.getPoint(), point);
        Vector3 p2ToPoint = Vector3.subtract(Vector3.add(ray.getPoint(), ray.vector), point);

        float areaOfTriangleTimesTwo = Vector3.crossProduct(p1ToPoint, p2ToPoint).length(); 
        float lengthOfBase = ray.getVector().length();
        
        float distanceFromPointToRay = areaOfTriangleTimesTwo / lengthOfBase;
        
        return distanceFromPointToRay;
    }

	@Override
	public int collidesWith(ICollider other, ContactCollection contacts) {
		return CollisionDetector.collidesWith(this, other, contacts);
	}
	
	public void setPoint(Vector3 point){
		this.point = point;
	}
	
	public Vector3 getPoint(){
		if( transform != null ){
			return Vector3.add(transform.getPosition(), point);
		} else{
			return point;
		}
	}
	
	public Vector3 getVector(){
		return vector; 
	}
	
	@Override
	public String getTag() {
		return TAG; 
	}	
	
	@Override 
	public int getColliderType(){
		return ICollider.COLLIDER_TYPE_RAY;
	}

	@Override
	public void setTransform(Transform transform) {
		this.transform = transform; 
	}

	@Override
	public Transform getTransform() {
		return transform; 
	}
}
