package com.wmp.nofrills3d.physics;

import com.wmp.nofrills3d.entities.SkinnedModel;
import com.wmp.nofrills3d.entities.Transform;
import com.wmp.nofrills3d.geometry.Bone;
import com.wmp.nofrills3d.math.Matrix4;
import com.wmp.nofrills3d.math.Quaternion;
import com.wmp.nofrills3d.math.Vector3;

/**
 * decorator that wraps a collidable and to be associated with a bone 
 * @author josh
 *
 */
public class BoneCollider extends Transform implements ICollider {	

	protected Transform transform; 
	protected ICollider collider = null;
	protected SkinnedModel skinnedModel = null; 
	protected int boneIndex = -1;
	protected Vector3 offset = Vector3.createZero();
	
	protected Quaternion wrkQuaternion = Quaternion.createIdentity();
	protected Matrix4 worldMatrix = Matrix4.createIdentity(); 
	
	public BoneCollider(ICollider collider, SkinnedModel skinnedModel, int boneIndex){
		this.collider = collider;		
		this.skinnedModel = skinnedModel; 
		this.boneIndex = boneIndex;
		
		transform = collider.getTransform();
		collider.setTransform(this);
		
		if( collider instanceof SphereCollider ){
			offset = ((SphereCollider)collider).centre;
		}
		else if( collider instanceof BoxCollider ){
			
		}
	}
	
	@Override
	public void update(float elapsedTime) {		
		Matrix4.multiply(skinnedModel.modelTransform, skinnedModel.worldTransforms[boneIndex], worldMatrix);
		
		wrkQuaternion.set(0.0f, 0.0f, 0.0f);				
		
		// take 1
		//float[] res = new float[4];				
		//Matrix.multiplyMV(res, 0, worldMatrix.matrix, 0, wrkQuaternion.quaternion, 0);
		
		// take 2		
		//decomposeMatrix();
		
		// take 3
		Vector3 res = Vector3.project(offset, worldMatrix);
		
		this.setPosition(res);
		
		collider.update(elapsedTime);
	}
	
	protected void decomposeMatrix(){
		worldMatrix.getTranslationAsVector3(position);
		worldMatrix.getRotationAsQuaternion(wrkQuaternion).toEulerAngles(rotation);
	}
	
	@Override
	public String getTag() {
		return collider.getTag(); 
	}
	
	public Bone getBone(){
		return skinnedModel.findBoneAtIndex(boneIndex);
	}
	
	@Override 
	public int getColliderType(){
		return collider.getColliderType();
	}

	@Override
	public int collidesWith(ICollider other, ContactCollection contacts) {
		return collider.collidesWith(other, contacts);
	}	
	
	public ICollider getCollider(){
		return collider; 
	}

	@Override
	public void setTransform(Transform transform) {
		this.transform = transform; 
	}

	@Override
	public Transform getTransform() {
		return transform; 
	}

}
