/**
 * BoundingBox.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 22, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.physics;

import com.wmp.nofrills3d.entities.Transform;
import com.wmp.nofrills3d.math.Vector3;
import com.wmp.nofrills3d.utils.Logger;

public class BoxCollider implements ICollider{	
	
	public static final String TAG = "BoxCollider"; 
	
	public Transform transform = null; 
	
	public Vector3 min; 
	
	public Vector3 max;
	
	public Vector3 size = null; 
	
	private Vector3 wrkMin = new Vector3(); 
	
	private Vector3 wrkMax = new Vector3(); 
	
	public BoxCollider(Transform owner, Vector3 min, Vector3 max)
    {
		this.transform = owner; 
        this.min = min;
        this.max = max;
        
        if( Logger.Logging ){
			Logger.LogTODO(TAG, "BoxCollider incomplete");
		}
    }	
	
	@Override
	public void update(float elapsedTime) {
		// TODO Auto-generated method stub
		
	}
	
//	public Vector3[] getCorners()
//    {
//        return new Vector3[] {
//            new Vector3(this.min.x, this.max.y, this.max.z), 
//            new Vector3(this.max.x, this.max.y, this.max.z),
//            new Vector3(this.max.x, this.min.y, this.max.z), 
//            new Vector3(this.min.x, this.min.y, this.max.z), 
//            new Vector3(this.min.x, this.max.y, this.min.z),
//            new Vector3(this.max.x, this.max.y, this.min.z),
//            new Vector3(this.max.x, this.min.y, this.min.z),
//            new Vector3(this.min.x, this.min.y, this.min.z)
//        };
//    }

	@Override
	public int collidesWith(ICollider other, ContactCollection contacts) {
		if( other instanceof BoxCollider ){
			Contact.ContainmentType containmentType = collidesWithBoxCollider((BoxCollider) other); 
			
			if( containmentType != Contact.ContainmentType.Disjoint ){
				Contact contact = new Contact(this, other, containmentType);
				contacts.addContact(contact); 
			}
			
			return 1; 
		}
		
		if( Logger.Logging ){
			Logger.LogTODO(TAG, "CollidesWith(Collidable, ContactCollection) incomplete");
		}		
		
		return 0; 
	}
	
	@Override
	public String getTag() {
		return TAG; 
	}
	
	@Override 
	public int getColliderType(){
		return ICollider.COLLIDER_TYPE_BOX;
	}
	
	protected Contact.ContainmentType collidesWithBoxCollider(BoxCollider box)
    {
		Vector3 min = getMin(); 
		Vector3 max = getMax();
		
		Vector3 boxmin = box.getMin(); 
		Vector3 boxmax = box.getMax(); 
		
        if (boxmax.x < min.x
            || boxmin.x > max.x
            || boxmax.y < min.y
            || boxmin.y > max.y
            || boxmax.z < min.z
            || boxmin.z > max.z)
            return Contact.ContainmentType.Disjoint;


        if (boxmin.x >= min.x
            && boxmax.x <= max.x
            && boxmin.y >= min.y
            && boxmax.y <= max.y
            && boxmin.z >= min.z
            && boxmax.z <= max.z)
            return Contact.ContainmentType.Contains;

        return Contact.ContainmentType.Intersects;
    }
	
	protected Contact.ContainmentType collidesWithPoint(Vector3 point)
    {
		Vector3 min = getMin(); 
		Vector3 max = getMax(); 
		
        if (point.x < min.x
            || point.x > max.x
            || point.y < min.y
            || point.y > max.y
            || point.z < min.z
            || point.z > max.z)
        {
            return Contact.ContainmentType.Disjoint;
        }
        else if (point.x == min.x
            || point.x == max.x
            || point.y == min.y
            || point.y == max.y
            || point.z == min.z
            || point.z == max.z)
            return Contact.ContainmentType.Intersects;
        else
            return Contact.ContainmentType.Contains;
    }

	@Override
	public void setTransform(Transform transform) {
		this.transform = transform; 
	}

	@Override
	public Transform getTransform() {
		return transform; 
	}	
	
	public Vector3 getSize(){
		if( size == null ){
			size = Vector3.createZero();
		}
		
		size.x = max.x - min.x * (transform != null ? transform.getScale().x : 1);
		size.y = max.y - min.y * (transform != null ? transform.getScale().y : 1);
		size.z = max.z - min.z * (transform != null ? transform.getScale().z : 1);		
		
		return size; 
	}
	
	public Vector3 getMin(){
		if( transform == null ){
			return this.min; 
		} else{
			return wrkMin.set(min).scale(transform.getScale()).add(transform.getPosition());
		}
	}
	
	public Vector3 getMax(){
		if( transform == null ){
			return this.max; 
		} else{
			return wrkMax.set(max).scale(transform.getScale()).add(transform.getPosition());
		}
	}
}
