package com.wmp.nofrills3d.physics;

import java.util.ArrayList;
import java.util.Iterator;

import com.wmp.nofrills3d.entities.Transform;
import com.wmp.nofrills3d.math.Vector3;

public class ColliderCollection implements Iterable<ICollider>, IColliderCollection {

	private final ArrayList<ICollider> colliders = new ArrayList<ICollider>();	
	
	public void update(float elapsedTime) {
		for(ICollider collider : colliders ){
			collider.update(elapsedTime);
		}
	}

	@Override
	public int collidesWith(ICollider other, ContactCollection contacts) {
		int contactCount = 0; 
		for(ICollider collider : colliders ){
			contactCount += collider.collidesWith(other, contacts);
		}
		
		return contactCount; 
	}

	@Override
	public Iterator<ICollider> iterator() {
		return colliders.iterator();
	}
	
	public void clear(){
		colliders.clear();
    }

    public void addCollider(ICollider collider){
        colliders.add(collider);
    }

    public int size(){
        return colliders.size();
    }
    
    public ICollider getCollidableAtIndex(int index){
        return colliders.get(index);
    }
    
    public ICollider getCollidableWithTag(String tag){
    	for(ICollider collider : colliders ){
			if( collider.getTag().equals(tag) ){
				return collider; 
			}
		}
    	return null; 
    }	
}
