package com.wmp.nofrills3d.physics;

import com.wmp.nofrills3d.math.Plane;
import com.wmp.nofrills3d.math.Ray;
import com.wmp.nofrills3d.math.Vector3;

public final class CollisionDetector {
	
	public static int collidesWith(ICollider colliderA, ICollider colliderB, ContactCollection contacts) {
		
		if( colliderA instanceof BoneCollider ){
			colliderA = ((BoneCollider)colliderA).getCollider();
		}
		
		if( colliderB instanceof BoneCollider ){
			colliderB = ((BoneCollider)colliderB).getCollider();
		}
		
		if( colliderA.getColliderType() == ICollider.COLLIDER_TYPE_RAY && colliderB.getColliderType() == ICollider.COLLIDER_TYPE_SPHERE ){
			return collidesWith((Ray)colliderA, (SphereCollider)colliderB, contacts);
		}
		else if( colliderA.getColliderType() == ICollider.COLLIDER_TYPE_SPHERE && colliderB.getColliderType() == ICollider.COLLIDER_TYPE_RAY ){
			return collidesWith((SphereCollider)colliderA, (Ray)colliderB, contacts);
		}
		
		return 0; 
	}
	
	protected static int collidesWith(SphereCollider colliderA, SphereCollider colliderB, ContactCollection contacts) {
		return 0; 
	}
	
	protected static int collidesWith(Ray colliderA, SphereCollider colliderB, ContactCollection contacts) {
		return CollisionDetector.collidesWith(colliderB, colliderA, contacts, true);
	} 
	
	protected static int collidesWith(SphereCollider colliderA, Ray colliderB, ContactCollection contacts) {
		return CollisionDetector.collidesWith(colliderA, colliderB, contacts, true);
	}
	
	protected static int collidesWith(SphereCollider sphere, Ray ray, ContactCollection contacts, boolean swap) {		
		
		if( ray.intersects(sphere) ){
			Contact contact = null; 
			if( swap ){
				contact = new Contact(sphere, ray);
				contact.contactPoint = Vector3.subtract(sphere.getCentre(), ray.getPoint()).normalise();
			} else{
				contact = new Contact(ray, sphere);
				contact.contactPoint = Vector3.subtract(ray.getPoint(), sphere.getCentre()).normalise().scale(sphere.radius).add(sphere.getCentre());;
			}			
			
			contact.distance = Vector3.distance(ray.getPoint(), contact.contactPoint); 
			
			contacts.addContact(contact);
			
			return 1;
		}
		
		return 0; 
	}
	
	public static Vector3 getIntersectionPoint(Ray ray, Plane plane){
		Vector3 rayToPlaneVector = new Vector3(); 		
		Vector3.subtract(ray.getPoint(), plane.point, rayToPlaneVector);
		
		float scaleFactor = Vector3.dotProduct(rayToPlaneVector, plane.normal) / Vector3.dotProduct(ray.getVector(), plane.normal);
		
		Vector3 scaledVector = ray.getVector().clone(); 
		scaledVector.scale(scaleFactor);		
		
		Vector3 intersectionPoint = ray.getPoint().clone();
		intersectionPoint.add(scaledVector);
		return intersectionPoint; 
	}
}
