/**
 * BoundingSphere.java (NoFrills3D)
 * Created by Joshua Newnham on Apr 22, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.physics;

import com.wmp.nofrills3d.entities.Transform;
import com.wmp.nofrills3d.math.Vector3;
import com.wmp.nofrills3d.utils.Logger;

public class SphereCollider implements ICollider {

	public static final String TAG = "SphereCollider";
	
	public Transform transform; 	
	public Vector3 centre = Vector3.createZero(); 
	public float radius = 0.0f;
	
	// tmp variable to reduce object creation 
	private Vector3 wrkCentre = Vector3.createZero();
	
	public SphereCollider(Transform transform){
		super(); 
		this.transform = transform; 
	}
	
	public SphereCollider(Transform transform, Vector3 centre, float radius){
		this.transform = transform; 
		this.centre = centre; 
		this.radius = radius; 
	}
	
	@Override
	public void update(float elapsedTime) {
		
	}
	
	@Override
	public int collidesWith(ICollider other, ContactCollection contacts) {
		if( other instanceof SphereCollider ){
			SphereCollider sphereCollider = (SphereCollider)other; 
			
			float distance = Vector3.distance(getCentre(), sphereCollider.getCentre());  
			float radii = getRadius() + sphereCollider.getRadius(); 
			if( distance < radii ){
				Contact contact = new Contact(this, other);
				contact.contactPoint = Vector3.subtract(sphereCollider.getCentre(), getCentre()).normalise().scale(getRadius()).add(getCentre());
				contact.distance = distance - radii;				
				contacts.addContact(contact);
				
				return 1; 
			}
		}
		if( Logger.Logging ){
			Logger.LogTODO(TAG, "collidesWith(Collidable, ContactCollection) incomplete");
		}
		
		return 0; 
	}	

	@Override
	public String getTag() {
		return TAG; 
	}	
	
	@Override 
	public int getColliderType(){
		return ICollider.COLLIDER_TYPE_SPHERE;
	}

	@Override
	public void setTransform(Transform transform) {
		this.transform = transform; 
	}

	@Override
	public Transform getTransform() {
		return transform; 
	}
	
	public Vector3 getCentre(){
		if( transform == null ){
			return this.centre; 
		} else{
			return wrkCentre.set(this.centre).add(transform.getPosition());
		}
	}
	
	public float getRadius(){
		if( transform == null ){
			return this.radius; 
		} else{
			return this.radius * transform.getScale().x; 
		}
	}
}
