/**
 * GLView.java (ColourExplosion)
 * Created by Joshua Newnham on Apr 9, 2013.
 * Copyright (c) 2013 We Make Play All rights reserved.
 */
package com.wmp.nofrills3d.scene;

import android.app.ActivityManager;
import android.content.Context;
import android.content.pm.ConfigurationInfo;
import android.opengl.GLSurfaceView;
import android.os.Build;
import android.util.AttributeSet;
import android.view.SurfaceHolder;

public class SceneView extends GLSurfaceView { 
	
	protected SceneRenderer renderer; 		
	
	/**
	 * @param context
	 */
	public SceneView(Context context) {
		super(context);
		init(); 
	}
	
	public SceneView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(); 
	}
	
	@Override
	public SurfaceHolder getHolder() {
		return super.getHolder(); 
	}
	
	protected void init(){
		if( supportsOpenGL2() ){						
			this.setEGLContextClientVersion(2);
            this.setRenderer(renderer = new SceneRenderer(this));
            
            this.setRenderMode(RENDERMODE_CONTINUOUSLY); 
		} else{
			// TODO handle exception 
		}
		
	}
	
	public boolean isGLInitilised(){
		return renderer.isGLInitilised(); 
	}
	
	public void onPause(){
		if( renderer != null ){
			renderer.onPause(); 
		}
		
		super.onPause(); 
	}
	
	public void onResume(){	
		super.onResume();
		
		if( renderer != null ){
			renderer.onResume(); 
		}					
	}	
	
	public void onDestory(){
		super.onDetachedFromWindow();
	}
	
	public void setFrameRate(long frameRate){
		this.renderer.frameRate = frameRate; 
	}		
	
	public SceneRenderer getSceneRenderer(){
		return renderer; 
	}
	
	public void setDelegate(ISceneRendererDelegate delegate){
		renderer.delegate = delegate; 		
	}
	
	public int getViewWidth(){
		return renderer.viewWidth;
	}
	
	public int getViewHeight(){
		return renderer.viewHeight; 
	}
	
	public void setCurrentScene(IScene scene){
		renderer.setCurrentScene(scene);
	}
	
	public IScene getCurrentScene(){
		return renderer.getCurrentScene();
	}
	
	public boolean supportsOpenGL2(){
		ActivityManager activityManager = (ActivityManager) getContext().getSystemService(Context.ACTIVITY_SERVICE);
        ConfigurationInfo configurationInfo = activityManager.getDeviceConfigurationInfo();        
		return configurationInfo.reqGlEsVersion >= 0x20000
                || (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH_MR1
                 && (Build.FINGERPRINT.startsWith("generic")
                  || Build.FINGERPRINT.startsWith("unknown")
                  || Build.MODEL.contains("google_sdk")
                  || Build.MODEL.contains("Emulator")
                  || Build.MODEL.contains("Android SDK built for x86")));
	}		

}
